Type: Package
Package: qtl2pleio
Title: Testing Pleiotropy in Multiparental Populations
Version: 1.4.4
Authors@R: 
    person(given = "Frederick J",
           family = "Boehm",
           role = c("aut", "cre"),
           email = "frederick.boehm@gmail.com",
           comment = c(ORCID = "0000-0002-1644-5931"))
Description: We implement an
    adaptation of Jiang & Zeng's (1995) <doi:10.1093/genetics/140.3.1111> likelihood ratio test for testing
    the null hypothesis of pleiotropy against the alternative hypothesis,
    two separate quantitative trait loci. The test differs from that in Jiang & Zeng (1995)
    and that in Tian et al. (2016) <doi:10.1534/genetics.115.183624> in
    that our test accommodates multiparental populations.
License: MIT + file LICENSE
URL: https://github.com/fboehm/qtl2pleio
BugReports: https://github.com/fboehm/qtl2pleio/issues
Depends: R (>= 3.2)
Imports: dplyr, gemma2, ggplot2, magrittr, MASS, Rcpp, rlang, tibble,
        parallel
Suggests: covr, mvtnorm, knitr, rmarkdown, testthat, broman, devtools,
        qtl2, parallelly
LinkingTo: Rcpp, RcppEigen
Encoding: UTF-8
Language: en-US
NeedsCompilation: yes
RoxygenNote: 7.3.3
Packaged: 2026-04-25 19:40:25 UTC; fred
Author: Frederick J Boehm [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-1644-5931>)
Maintainer: Frederick J Boehm <frederick.boehm@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-25 23:30:02 UTC
Built: R 4.5.2; aarch64-apple-darwin20; 2026-04-26 23:59:19 UTC; unix
Archs: qtl2pleio.so.dSYM

# Generated by roxygen2: do not edit by hand

export("%>%")
export(add_pmap)
export(boot_pvl)
export(calc_Bhat)
export(calc_Sigma)
export(calc_covs)
export(calc_lrt_tib)
export(calc_profile_lods)
export(check_identical)
export(check_missingness)
export(convert_to_scan1_output)
export(find_pleio_peak_tib)
export(fit1_pvl)
export(get_effects)
export(make_id2keep)
export(plot_pvl)
export(prep_X_list)
export(prep_mytab)
export(rcpp_calc_Bhat)
export(rcpp_calc_Bhat2)
export(rcpp_log_dmvnorm2)
export(scan_multi_onechr)
export(scan_multi_oneqtl)
export(scan_multi_oneqtl_perm)
export(scan_pvl)
export(sim1)
export(subset_input)
export(subset_kinship)
importFrom(Rcpp,sourceCpp)
importFrom(magrittr,"%>%")
importFrom(rlang,.data)
importFrom(stats,complete.cases)
importFrom(stats,profile)
importFrom(stats,var)
useDynLib(qtl2pleio, .registration = TRUE)

<!-- README.md is generated from README.Rmd. Please edit README.Rmd -->

# qtl2pleio

[![CRAN\_Status\_Badge](https://www.r-pkg.org/badges/version/qtl2pleio)](https://cran.r-project.org/package=qtl2pleio)
[![Binder](https://static.mybinder.org/badge_logo.svg)](https://mybinder.org/v2/gh/fboehm/qtl2pleio/master?urlpath=rstudio)
[![Travis-CI Build
Status](https://travis-ci.org/fboehm/qtl2pleio.svg?branch=master)](https://travis-ci.org/fboehm/qtl2pleio)
[![Coverage
Status](https://img.shields.io/codecov/c/github/fboehm/qtl2pleio/master.svg)](https://codecov.io/github/fboehm/qtl2pleio?branch=master)
[![Project Status: Active – The project has reached a stable, usable
state and is being actively
developed.](https://www.repostatus.org/badges/latest/active.svg)](https://www.repostatus.org/#active)
[![status](https://joss.theoj.org/papers/66bca5dc3d2e72b6259159bad07aafaf/status.svg)](https://joss.theoj.org/papers/10.21105/joss.01435)
[![DOI](https://zenodo.org/badge/104493705.svg)](https://zenodo.org/badge/latestdoi/104493705)

## Overview

`qtl2pleio` is a software package for use with the [R statistical
computing environment](https://cran.r-project.org/). `qtl2pleio` is
freely available for download and use. I share it under the [MIT
license](https://opensource.org/licenses/mit-license.php). The user will
also want to download and install the [`qtl2` R
package](https://kbroman.org/qtl2/).

Click
[here](https://mybinder.org/v2/gh/fboehm/qtl2pleio/master?urlpath=rstudio)
to explore `qtl2pleio` within a live [Rstudio](https://rstudio.com/)
session in “the cloud”.

## Contributor guidelines

We eagerly welcome contributions to `qtl2pleio`. All pull requests will
be considered. Features requests and bug reports may be filed as Github
issues. All contributors must abide by the [code of
conduct](https://github.com/fboehm/qtl2pleio/blob/master/CONDUCT.md).

## Technical support

For technical support, please open a Github issue. If you’re just
getting started with `qtl2pleio`, please examine the
[vignettes](https://fboehm.us/software/qtl2pleio) on the [package’s web
site](https://fboehm.us/software/qtl2pleio). You can also email
<frederick.boehm@gmail.com> for assistance.

## Goals

The goal of `qtl2pleio` is, for a pair of traits that show evidence for
a QTL in a common region, to distinguish between pleiotropy (the null
hypothesis, that they are affected by a common QTL) and the alternative
that they are affected by separate QTL. It extends the likelihood ratio
test of [Jiang and Zeng
(1995)](https://www.genetics.org/content/140/3/1111.long) for
multiparental populations, such as Diversity Outbred mice, including the
use of multivariate polygenic random effects to account for population
structure. `qtl2pleio` data structures are those used in the
[`rqtl/qtl2`](https://kbroman.org/qtl2/) package.

## Installation

To install qtl2pleio, use `install_github()` from the
[devtools](https://devtools.r-lib.org) package.

``` r
install.packages("qtl2pleio")
```

You may also wish to install the [R/qtl2](https://kbroman.org/qtl2/). We
will use it below.

``` r
install.packages("qtl2")
```

## Example

Below, we walk through an example analysis with Diversity Outbred mouse
data. We need a number of preliminary steps before we can perform our
test of pleiotropy vs. separate QTL. Many procedures rely on the R
package `qtl2`. We first load the `qtl2` and `qtl2pleio` packages.

``` r
library(qtl2)
library(qtl2pleio)
library(ggplot2)
```

### Reading data from `qtl2data` repository on github

We’ll consider the
[`DOex`](https://github.com/rqtl/qtl2data/tree/master/DOex/) data in the
[`qtl2data`](https://github.com/rqtl/qtl2data/) repository. We’ll
download the DOex.zip file before calculating founder allele dosages.

``` r
file <- paste0("https://raw.githubusercontent.com/rqtl/",
               "qtl2data/master/DOex/DOex.zip")
DOex <- read_cross2(file)
```

``` r
probs <- calc_genoprob(DOex)
```

Let’s calculate the founder allele dosages from the 36-state genotype
probabilities.

``` r
pr <- genoprob_to_alleleprob(probs)
```

We now have an allele probabilities object stored in `pr`.

``` r
names(pr)
#> [1] "2" "3" "X"
dim(pr$`2`)
#> [1] 261   8 127
```

We see that `pr` is a list of 3 three-dimensional arrays - one array for
each of 3 chromosomes.

### Kinship calculations

For our statistical model, we need a kinship matrix. We get one with the
`calc_kinship` function in the `rqtl/qtl2` package.

``` r
kinship <- calc_kinship(probs = pr, type = "loco")
```

### Statistical model specification

We use the multivariate linear mixed effects model:

\[vec(Y) = X vec(B) + vec(G) + vec(E)\]

where \(Y\) contains phenotypes, X contains founder allele probabilities
and covariates, and B contains founder allele effects. G is the
polygenic random effects, while E is the random errors. We provide more
details in the vignette.

### Simulating phenotypes with `qtl2pleio::sim1`

The function to simulate phenotypes in `qtl2pleio` is `sim1`.

``` r
# set up the design matrix, X
pp <- pr[[2]] #we'll work with Chr 3's genotype data
```

``` r
#Next, we prepare a design matrix X
X <- gemma2::stagger_mats(pp[ , , 50], pp[ , , 50])
```

``` r
# assemble B matrix of allele effects
B <- matrix(data = c(-1, -1, -1, -1, 1, 1, 1, 1, -1, -1, -1, -1, 1, 1, 1, 1), nrow = 8, ncol = 2, byrow = FALSE)
# set.seed to ensure reproducibility
set.seed(2018-01-30)
Sig <- calc_Sigma(Vg = diag(2), Ve = diag(2), kinship = kinship[[2]])
# call to sim1
Ypre <- sim1(X = X, B = B, Sigma = Sig)
Y <- matrix(Ypre, nrow = 261, ncol = 2, byrow = FALSE)
rownames(Y) <- rownames(pp)
colnames(Y) <- c("tr1", "tr2")
```

Let’s perform univariate QTL mapping for each of the two traits in the Y
matrix.

``` r
s1 <- scan1(genoprobs = pr, pheno = Y, kinship = kinship)
```

Here is a plot of the results.

<img src="https://raw.githubusercontent.com/fboehm/qtl2pleio/master/man/figures/README-1d-lod-plots-1.png" width="100%" />

And here are the observed QTL peaks with LOD \> 8.

``` r
find_peaks(s1, map = DOex$pmap, threshold=8)
#>   lodindex lodcolumn chr      pos       lod
#> 1        1       tr1   3 82.77806 20.703383
#> 2        2       tr2   3 82.77806 14.417924
#> 3        2       tr2   X 48.10040  8.231551
```

### Perform two-dimensional scan as first step in pleiotropy vs. separate QTL hypothesis test

We now have the inputs that we need to do a pleiotropy vs. separate QTL
test. We have the founder allele dosages for one chromosome, *i.e.*, Chr
3, in the R object `pp`, the matrix of two trait measurements in `Y`,
and a LOCO-derived kinship matrix, `kinship[[2]]`.

``` r
out <- suppressMessages(scan_pvl(probs = pp,
                pheno = Y,
                kinship = kinship[[2]], # 2nd entry in kinship list is Chr 3
                start_snp = 38,
                n_snp = 25
                ))
```

### Create a profile LOD plot to visualize results of two-dimensional scan

To visualize results from our two-dimensional scan, we calculate profile
LOD for each trait. The code below makes use of the R package `ggplot2`
to plot profile LODs over the scan region.

``` r
library(dplyr)
out %>%
  calc_profile_lods() %>%
  add_pmap(pmap = DOex$pmap$`3`) %>%
  ggplot() + geom_line(aes(x = marker_position, y = profile_lod, colour = trait))
```

<img src="man/figures/README-profile-plot-1.png" width="100%" />

### Calculate the likelihood ratio test statistic for pleiotropy v separate QTL

We use the function `calc_lrt_tib` to calculate the likelihood ratio
test statistic value for the specified traits and specified genomic
region.

``` r
(lrt <- calc_lrt_tib(out))
#> [1] 0
```

### Bootstrap analysis to get p-values

Before we call `boot_pvl`, we need to identify the index (on the
chromosome under study) of the marker that maximizes the likelihood
under the pleiotropy constraint. To do this, we use the `qtl2pleio`
function `find_pleio_peak_tib`.

``` r
(pleio_index <- find_pleio_peak_tib(out, start_snp = 38))
#> log10lik13 
#>         50
```

``` r
set.seed(2018-11-25) # set for reproducibility purposes.
b_out <- suppressMessages(boot_pvl(probs = pp,
         pheno = Y,
         pleio_peak_index = pleio_index,
         kinship = kinship[[2]], # 2nd element in kinship list is Chr 3
         nboot = 10,
         start_snp = 38,
         n_snp = 25
         ))
```

``` r
(pvalue <- mean(b_out >= lrt))
#> [1] 1
```

## Citation information

``` r
citation("qtl2pleio")
#> 
#> To cite qtl2pleio in publications use:
#> 
#>   Boehm FJ, Chesler EJ, Yandell BS, Broman KW (2019) Testing pleiotropy
#>   vs. separate QTL in multiparental populations G3
#>   https://www.g3journal.org/content/9/7/2317
#> 
#> A BibTeX entry for LaTeX users is
#> 
#>   @Article{Boehm2019testing,
#>     title = {Testing pleiotropy vs. separate QTL in multiparental populations},
#>     author = {Frederick J. Boehm and Elissa J. Chesler and Brian S. Yandell and Karl W. Broman},
#>     journal = {G3},
#>     year = {2019},
#>     volume = {9},
#>     issue = {7},
#>     url = {https://www.g3journal.org/content/9/7/2317},
#>     eprint = {https://www.g3journal.org/content/ggg/9/7/2317.full.pdf},
#>   }
```

<!-- README.md is generated from README.Rmd. Please edit README.Rmd -->

# qtl2pleio

[![R-CMD-check](https://github.com/fboehm/qtl2pleio/workflows/R-CMD-check/badge.svg)](https://github.com/fboehm/qtl2pleio/actions)
[![CRAN_Status_Badge](https://www.r-pkg.org/badges/version/qtl2pleio)](https://cran.r-project.org/package=qtl2pleio)
[![Coverage
Status](https://img.shields.io/codecov/c/github/fboehm/qtl2pleio/master.svg)](https://app.codecov.io/github/fboehm/qtl2pleio?branch=master)
[![Project Status: Active – The project has reached a stable, usable
state and is being actively
developed.](https://www.repostatus.org/badges/latest/active.svg)](https://www.repostatus.org/#active)
[![status](https://joss.theoj.org/papers/66bca5dc3d2e72b6259159bad07aafaf/status.svg)](https://joss.theoj.org/papers/10.21105/joss.01435)
[![DOI](https://zenodo.org/badge/104493705.svg)](https://zenodo.org/badge/latestdoi/104493705)

## Overview

`qtl2pleio` is a software package for use with the [R statistical
computing environment](https://cran.r-project.org/). `qtl2pleio` is
freely available for download and use. I share it under the [MIT
license](https://opensource.org/licenses/mit-license.php). The user will
also want to download and install the [`qtl2` R
package](https://kbroman.org/qtl2/).

## Contributor guidelines

We eagerly welcome contributions to `qtl2pleio`. All pull requests will
be considered. Features requests and bug reports may be filed as Github
issues. All contributors must abide by the [code of
conduct](https://github.com/fboehm/qtl2pleio/blob/master/CONDUCT.md).

## Technical support

For technical support, please open a Github issue. If you’re just
getting started with `qtl2pleio`, please examine the vignettes. You can
also email <frederick.boehm@sdstate.edu> for assistance.

## Goals

The goal of `qtl2pleio` is, for a pair of traits that show evidence for
a QTL in a common region, to distinguish between pleiotropy (the null
hypothesis, that they are affected by a common QTL) and the alternative
that they are affected by separate QTL. It extends the likelihood ratio
test of [Jiang and Zeng
(1995)](https://doi.org/10.1093/genetics/140.3.1111) for multiparental
populations, such as Diversity Outbred mice, including the use of
multivariate polygenic random effects to account for population
structure. `qtl2pleio` data structures are those used in the
[`rqtl/qtl2`](https://kbroman.org/qtl2/) package.

## Installation

To install qtl2pleio, use `install_github()` from the
[devtools](https://devtools.r-lib.org) package.

``` r
install.packages("qtl2pleio")
```

You may also wish to install the [R/qtl2](https://kbroman.org/qtl2/). We
will use it below.

``` r
install.packages("qtl2")
```

## Example

Below, we walk through an example analysis with Diversity Outbred mouse
data. We need a number of preliminary steps before we can perform our
test of pleiotropy vs. separate QTL. Many procedures rely on the R
package `qtl2`. We first load the `qtl2` and `qtl2pleio` packages.

``` r
library(qtl2)
library(qtl2pleio)
library(ggplot2)
```

### Reading data from `qtl2data` repository on github

We’ll consider the
[`DOex`](https://github.com/rqtl/qtl2data/tree/master/DOex/) data in the
[`qtl2data`](https://github.com/rqtl/qtl2data/) repository. We’ll
download the DOex.zip file before calculating founder allele dosages.

``` r
file <- paste0("https://raw.githubusercontent.com/rqtl/",
               "qtl2data/master/DOex/DOex.zip")
DOex <- read_cross2(file)
```

``` r
probs <- calc_genoprob(DOex)
```

Let’s calculate the founder allele dosages from the 36-state genotype
probabilities.

``` r
pr <- genoprob_to_alleleprob(probs)
```

We now have an allele probabilities object stored in `pr`.

``` r
names(pr)
#> [1] "2" "3" "X"
dim(pr$`2`)
#> [1] 261   8 127
```

We see that `pr` is a list of 3 three-dimensional arrays - one array for
each of 3 chromosomes.

### Kinship calculations

For our statistical model, we need a kinship matrix. We get one with the
`calc_kinship` function in the `rqtl/qtl2` package.

``` r
kinship <- calc_kinship(probs = pr, type = "loco")
```

### Statistical model specification

We use the multivariate linear mixed effects model:

$$
\text{vec}(Y) = X \text{vec}(B) + \text{vec}(G) + \text{vec}(E)
$$

where $Y$ contains phenotypes, X contains founder allele probabilities
and covariates, and B contains founder allele effects. G is the
polygenic random effects, while E is the random errors. We provide more
details in the vignette.

### Simulating phenotypes with `qtl2pleio::sim1`

The function to simulate phenotypes in `qtl2pleio` is `sim1`.

``` r
# set up the design matrix, X
pp <- pr[[2]] #we'll work with Chr 3's genotype data
```

``` r
#Next, we prepare a design matrix X
X <- gemma2::stagger_mats(pp[ , , 50], pp[ , , 50])
```

``` r
# assemble B matrix of allele effects
B <- matrix(data = c(-1, -1, -1, -1, 1, 1, 1, 1, -1, -1, -1, -1, 1, 1, 1, 1), nrow = 8, ncol = 2, byrow = FALSE)
# set.seed to ensure reproducibility
set.seed(2018-01-30)
Sig <- calc_Sigma(Vg = diag(2), Ve = diag(2), kinship = kinship[[2]])
# call to sim1
Ypre <- sim1(X = X, B = B, Sigma = Sig)
Y <- matrix(Ypre, nrow = 261, ncol = 2, byrow = FALSE)
rownames(Y) <- rownames(pp)
colnames(Y) <- c("tr1", "tr2")
```

Let’s perform univariate QTL mapping for each of the two traits in the Y
matrix.

``` r
s1 <- scan1(genoprobs = pr, pheno = Y, kinship = kinship)
```

Here is a plot of the results.

<img src="https://raw.githubusercontent.com/fboehm/qtl2pleio/master/man/figures/README-1d-lod-plots-1.png" alt="" width="100%" />

And here are the observed QTL peaks with LOD \> 8.

``` r
find_peaks(s1, map = DOex$pmap, threshold=8)
#>   lodindex lodcolumn chr       pos       lod
#> 1        1       tr1   3  82.77806 16.287629
#> 2        2       tr2   3  82.77806 15.035673
#> 3        2       tr2   X 128.06510  9.007418
```

### Perform two-dimensional scan as first step in pleiotropy vs. separate QTL hypothesis test

We now have the inputs that we need to do a pleiotropy vs. separate QTL
test. We have the founder allele dosages for one chromosome, *i.e.*, Chr
3, in the R object `pp`, the matrix of two trait measurements in `Y`,
and a LOCO-derived kinship matrix, `kinship[[2]]`.

``` r
out <- suppressMessages(scan_pvl(probs = pp,
                pheno = Y,
                kinship = kinship[[2]], # 2nd entry in kinship list is Chr 3
                start_snp = 38,
                n_snp = 25
                ))
```

### Create a profile LOD plot to visualize results of two-dimensional scan

To visualize results from our two-dimensional scan, we calculate profile
LOD for each trait. The code below makes use of the R package `ggplot2`
to plot profile LODs over the scan region.

``` r
library(dplyr)
out %>%
  calc_profile_lods() %>%
  add_pmap(pmap = DOex$pmap$`3`) %>%
  ggplot() + geom_line(aes(x = marker_position, y = profile_lod, colour = trait))
```

<img src="man/figures/README-profile-plot-1.png" alt="" width="100%" />

### Calculate the likelihood ratio test statistic for pleiotropy v separate QTL

We use the function `calc_lrt_tib` to calculate the likelihood ratio
test statistic value for the specified traits and specified genomic
region.

``` r
(lrt <- calc_lrt_tib(out))
#> [1] 0
```

### Bootstrap analysis to get p-values

Before we call `boot_pvl`, we need to identify the index (on the
chromosome under study) of the marker that maximizes the likelihood
under the pleiotropy constraint. To do this, we use the `qtl2pleio`
function `find_pleio_peak_tib`.

``` r
(pleio_index <- find_pleio_peak_tib(out, start_snp = 38))
#> log10lik13 
#>         50
```

``` r
set.seed(2018-11-25) # set for reproducibility purposes.
b_out <- suppressMessages(boot_pvl(probs = pp,
         pheno = Y,
         pleio_peak_index = pleio_index,
         kinship = kinship[[2]], # 2nd element in kinship list is Chr 3
         nboot = 10,
         start_snp = 38,
         n_snp = 25
         ))
```

``` r
(pvalue <- mean(b_out >= lrt))
#> [1] 1
```

## Citation information

``` r
citation("qtl2pleio")
#> To cite qtl2pleio in publications use:
#> 
#>   Boehm FJ, Chesler EJ, Yandell BS, Broman KW (2019) Testing pleiotropy
#>   vs. separate QTL in multiparental populations. G3 9:2317-2324
#>   doi:10.1534/g3.119.400098
#> 
#> A BibTeX entry for LaTeX users is
#> 
#>   @Article{Boehm2019testing,
#>     title = {Testing pleiotropy vs. separate QTL in multiparental populations},
#>     author = {Frederick J. Boehm and Elissa J. Chesler and Brian S. Yandell and Karl W. Broman},
#>     journal = {G3},
#>     year = {2019},
#>     volume = {9},
#>     issue = {7},
#>     pages = {2317-2324},
#>     doi = {10.1093/genetics/140.3.1111},
#>     url = {https://doi.org/10.1534/g3.119.400098},
#>     eprint = {https://academic.oup.com/g3journal/article-pdf/9/7/2317/40667800/g3journal2317.pdf},
#>   }
```

#' Calculate the phenotypes covariance matrix Sigma
#'
#' @param Vg d by d genetic covariance matrix for the d phenotypes
#' @param Ve d by d error covariance matrix for the d phenotypes
#' @param kinship optional n by n kinship matrix. if NULL, Vg is not used.
#' @param n_mouse number of subjects
#' @export
#' @return dn by dn covariance matrix
calc_Sigma <- function(Vg, Ve, kinship = NULL, n_mouse = nrow(kinship)) {
    if (!is.null(kinship)){
      out <- Vg %x% kinship + Ve %x% diag(n_mouse)
    }
    if (is.null(kinship)){
      out <- Ve %x% diag(n_mouse) # n_mouse must be specified when kinship = NULL
    }
    return(out)
}

library(qtl2pleio)
library(testthat)
context("Simulation of one phenotype vector")

X1 <- as.matrix(rbinom(n = 100, size = 1, prob = 1 / 2))
X2 <- as.matrix(rbinom(n = 100, size = 1, prob = 1 / 2))
gemma2::stagger_mats(X1, X2) -> X
Sigma <- calc_Sigma(Vg = diag(1, 2), Ve = diag(1, 2), kinship = diag(100))


test_that("sim1 outputs numeric vector", {
  expect_true(is.numeric(sim1(X = X, B = c(1, 2), Sigma = Sigma)))
})

#' Perform model fitting for all ordered pairs of markers in a genomic region of interest
#'
#' `scan_pvl` calculates log likelihood for d-variate phenotype model fits. Inputted parameter `start_snp` indicates where in the `probs` object to start the scan.
#'
#' The function first discards individuals with one or more missing phenotypes or missing covariates.
#' It then infers variance components, Vg and Ve. Both Vg and Ve
#' are d by d covariance matrices. It uses an expectation maximization algorithm, as
#' implemented in the `gemma2` R package. `gemma2` R package is an R implementation of the
#' GEMMA algorithm for multivariate variance component estimation (Zhou & Stephens 2014 Nature methods).
#' Note that variance components are fitted on a model that uses the d-variate phenotype
#' but contains no genetic information. This model does, however,
#' use the specified covariates (after dropping dependent columns
#' in the covariates matrix).
#' These inferred covariance matrices, \eqn{\hat{Vg}} and \eqn{\hat{Ve}},
#' are then used in subsequent model fitting via
#' generalized least squares.
#' Generalized least squares model fitting is applied to every d-tuple of
#' markers within the specified genomic region for `scan_pvl`.
#' For a single d-tuple of markers, we fit the model:
#' \deqn{vec(Y) = Xvec(B) + vec(G) + vec(E)} where
#' \deqn{G \sim MN(0, K, \hat{Vg})} and \deqn{E \sim MN(0, I, \hat{Ve})} where \eqn{MN} denotes the matrix-variate
#' normal distribution with three parameters: mean matrix, covariance among rows, and
#' covariance among columns. \eqn{vec} denotes the vectorization operation, ie, stacking by columns.
#' \eqn{K} is a kinship matrix, typically calculated by leave-one-chromosome-out methods.
#' \eqn{Y} is the n by d phenotypes matrix. \eqn{X} is a block-diagonal nd by fd matrix consisting of
#' d blocks each of dimension n by f. Each n by f block (on the diagonal) contains a matrix of
#' founder allele probabilities for the n subjects at a single marker. The off-diagonal blocks
#' have only zero entries.
#' The log-likelihood is returned for each model. The outputted object is a tibble with
#' d + 1 columns. The first d columns specify the markers used in the corresponding model fit, while
#' the last column specifies the log-likelihood value at that d-tuple of markers.
#'
#' @param probs an array of founder allele probabilities for a single chromosome
#' @param pheno a matrix of phenotypes
#' @param kinship a kinship matrix for one chromosome
#' @param addcovar a matrix, n subjects by c additive covariates
#' @param start_snp index of where to start the scan within probs
#' @param n_snp the number of (consecutive) markers to include in the scan
#' @param max_iter maximum number of iterations for EM algorithm
#' @param max_prec stepwise precision for EM algorithm. EM stops once incremental difference in log likelihood is less than max_prec
#' @param cores number of cores to use when parallelizing via parallel::mclapply. Set to 1 for no parallelization.
#' @export
#' @importFrom stats var
#' @references Knott SA, Haley CS (2000) Multitrait
#' least squares for quantitative trait loci detection.
#' Genetics 156: 899–911.
#'
#' Jiang C, Zeng ZB (1995) Multiple trait analysis
#' of genetic mapping for quantitative trait loci.
#' Genetics 140: 1111-1127.
#'
#' Zhou X, Stephens M (2014) Efficient multivariate linear
#' mixed model algorithms for genome-wide association studies.
#' Nature methods 11:407-409.
#'
#' Broman KW, Gatti DM, Simecek P, Furlotte NA, Prins P, Sen S, Yandell BS, Churchill GA (2019)
#' R/qtl2: software for mapping quantitative trait loci with high-dimensional data and
#' multi-parent populations. GENETICS https://www.genetics.org/content/211/2/495.
#'
#'
#' @examples
#' # read data
#' n <- 50
#' pheno <- matrix(rnorm(2 * n), ncol = 2)
#' rownames(pheno) <- paste0("s", 1:n)
#' colnames(pheno) <- paste0("tr", 1:2)
#' probs <- array(dim = c(n, 2, 5))
#' probs[ , 1, ] <- rbinom(n * 5, size = 1, prob = 0.2)
#' probs[ , 2, ] <- 1 - probs[ , 1, ]
#' rownames(probs) <- paste0("s", 1:n)
#' colnames(probs) <- LETTERS[1:2]
#' dimnames(probs)[[3]] <- paste0("m", 1:5)
#' scan_pvl(probs = probs, pheno = pheno, kinship = NULL,
#' start_snp = 1, n_snp = 5, cores = 1)
#' @importFrom rlang .data
#' @return a tibble with d + 1 columns. First d columns indicate the genetic data (by listing the marker ids) used in the design matrix; last is log10 likelihood

scan_pvl <- function(probs,
                     pheno,
                     kinship = NULL,
                     addcovar = NULL,
                     start_snp = 1,
                     n_snp,
                     max_iter = 1e+04,
                     max_prec = 1 / 1e+08,
                     cores = 1
                     )
    {
    inputs <- process_inputs(probs = probs,
                             pheno = pheno,
                             addcovar = addcovar,
                             kinship = kinship,
                             max_iter = max_iter,
                             max_prec = max_prec)
    # prepare table of marker indices for each call of scan_pvl
    d_size <- ncol(inputs$pheno)
    mytab <- prep_mytab(d_size = d_size, n_snp = n_snp)
    # set up parallel analysis
    out <- scan_pvl_clean(mytab = mytab,
                          addcovar = inputs$addcovar,
                          probs = inputs$probs,
                          Sigma_inv = inputs$Sigma_inv,
                          Sigma = inputs$Sigma,
                          start_snp = start_snp,
                          pheno = inputs$pheno,
                          n_snp = n_snp,
                          cores = cores
                          )
    return(out)
}



scan_pvl_clean <- function(pheno,
                           probs,
                           addcovar,
                           Sigma_inv,
                           Sigma,
                           start_snp,
                           mytab,
                           n_snp,
                           cores = 1){
    list_result <- parallel::mclapply(X = as.data.frame(t(mytab)),
                                     FUN = fit1_pvl,
                                     addcovar = addcovar,
                                     probs = probs,
                                     inv_S = Sigma_inv,
                                     S = Sigma,
                                     start_snp = start_snp,
                                     pheno = pheno,
                                     mc.cores = cores
                                     )
    mytab$loglik <- unlist(list_result)
    marker_id <- dimnames(probs)[[3]][start_snp:(start_snp + n_snp - 1)]
    mytab2 <- tibble::as_tibble(apply(FUN = function(x) marker_id[x],
                                      X = mytab[, -ncol(mytab)],
                                      MARGIN = 2))
    mytab2$log10lik <- mytab$loglik / log(10)
    return(mytab2)
}

library(qtl2pleio)
library(testthat)
context("testing scan_pvl")

# setup
# read data
is_inst <- function(pkg) {
  nzchar(system.file(package = pkg))
}
qtl2_indic <- is_inst("qtl2")

if (qtl2_indic) {
  iron <- qtl2::read_cross2(system.file("extdata", "iron.zip", package="qtl2"))
  # insert pseudomarkers into map
  map <- qtl2::insert_pseudomarkers(iron$gmap, step=1)
  # calculate genotype probabilities
  probs <- qtl2::calc_genoprob(iron, map, error_prob=0.002)
  # grab phenotypes and covariates; ensure that covariates have names attribute
  pheno <- iron$pheno
  # leave-one-chromosome-out kinship matrices
  kinship <- qtl2::calc_kinship(probs, "loco")$`1`
  # get founder allele probabilites
  probs <- qtl2::genoprob_to_alleleprob(probs)$`1`
  ac <- matrix(as.numeric(iron$covar$sex == "m", ncol = 1))
  colnames(ac) <- "sex"
  rownames(ac) <- rownames(probs)

  ## first scan_pvl call
  scan_out <- scan_pvl(probs = probs,
                       pheno = pheno,
                       kinship = kinship,
                       start_snp = 1,
                       n_snp = 10, cores = 1
                       )

  test_that("scan_pvl returns a dataframe where the last column has numeric entries, all negative", {
    expect_true(identical(rep(TRUE, nrow(scan_out)),
                          as.vector(scan_out[ , ncol(scan_out)] < 0)))
    expect_true(is.data.frame(scan_out))
  })

  so_cov <- scan_pvl(probs = probs,
                     pheno = pheno,
                     addcovar = ac,
                     kinship = kinship,
                     start_snp = 1,
                     n_snp = 10, cores = 1
                     )



  foo <- scan_pvl(probs = probs,
           pheno = pheno,
           kinship = NULL,
           start_snp = 1,
           n_snp = 10, cores = 1
  )

  test_that("scan_pvl handles missing values in covariates appropriately", {
    expect_equal(sum(!is.na(scan_out)), prod(dim(scan_out)))
    expect_equal(sum(!is.na(so_cov)), prod(dim(so_cov)))
  })

  test_that("scan_pvl handles kinship = NULL", {
    expect_true(is.data.frame(foo))
  })



  test_that("scan_pvl throws error when phenotypes matrix is not full rank",{
    pheno_singular <- cbind(pheno[ , 1], 2 * pheno[ , 1])
    expect_error(scan_pvl(probs = probs,
                          pheno = pheno_singular,
                          kinship = NULL,
                          start_snp = 1,
                          n_snp = 10, cores = 1
    ),
    regexp = "Phenotypes matrix is not full rank")
  })
}

#' Prepare mytab object for use within scan_pvl R code
#'
#' @param d_size an integer, the number of traits
#' @param n_snp an integer, the number of markers
#' @param pvl logical indicating whether to output dataframe with all d-tuples for a d-QTL scan, or only those models that examine one marker at a time.
#' @export
#' @return a data.frame with d_size + 1 columns and (n_snp)^d_size rows. Last column is NA and named loglik.
#' @examples
#' prep_mytab(2, 10)
prep_mytab <- function(d_size, n_snp, pvl = TRUE) {
    mytab <- expand.grid(rep(list(1:n_snp), d_size))
    if (!pvl) {
        mytab <- mytab %>%
            dplyr::filter_all(dplyr::all_vars(. == Var1))
    }
    mytab$loglik <- NA
    return(mytab)
}


#' Create a list of component X matrices for input to stagger_mats, to ultimately create design matrix
#'
#' @param indices a vector of integers
#' @param start_snp an integer denoting the index (within genotype probabilities array) where the scan should start
#' @param probs a three-dimensional array of genotype probabilities for a single chromosome
#' @param covariates a matrix of covariates
#' @export
#' @return a list of design matrices, ultimately useful when constructing the (multi-locus) design matrix
#' @examples
#' pp <- array(rbinom(n = 200, size = 1, prob = 0.5), dim = c(10, 2, 10))
#' prep_X_list(1:3, 1, probs = pp, covariates = NULL)
prep_X_list <- function(indices, start_snp, probs, covariates) {
    indices <- as.numeric(indices)
    if (!is.null(covariates)) {
        out <- lapply(X = as.list(indices), FUN = function(x) {
            index <- x + start_snp - 1
            foo <- cbind(probs[, , index], covariates)
            return(foo)
        })
    } else {
        out <- lapply(X = as.list(indices), FUN = function(x) {
            index <- x + start_snp - 1
            foo <- probs[, , index]
            return(foo)
        })
    }
    return(out)
}

library(qtl2pleio)
context("testing prep_mytab output's structure")


test_that("prep_mytab produces output with correct dimensions", {
  expect_equal(ncol(prep_mytab(d_size = 3, n_snp = 4)), 4)
  expect_equal(ncol(prep_mytab(d_size = 4, n_snp = 4)), 5)
  expect_equal(nrow(prep_mytab(d_size = 3, n_snp = 4)), 4 ^ 3)
  expect_equal(nrow(prep_mytab(d_size = 4, n_snp = 4)), 4 ^ 4)
})


test_that("verify that last column is all NAs", {
  expect_identical(as.vector(prep_mytab(d_size = 2, n_snp = 4)[, 3]),
                   rep(NA, 4 ^ 2))
})

#' Perform bootstrap sampling and calculate test statistic for each bootstrap sample
#'
#' Create a bootstrap sample, perform multivariate QTL scan, and calculate log10 LRT statistic
#'
#' Performs a parametric bootstrap method to calibrate test statistic values in the test of
#' pleiotropy vs. separate QTL. It begins by inferring parameter values at
#' the `pleio_peak_index` index value in the object `probs`. It then uses
#' these inferred parameter values in sampling from a multivariate normal
#' distribution. For each of the `nboot` sampled phenotype vectors, a two-dimensional QTL
#' scan, starting at the marker indexed by `start_snp` within the object
#' `probs` and extending for a total of `n_snp` consecutive markers. The
#' two-dimensional scan is performed via the function `scan_pvl_clean`. For each
#' two-dimensional scan, a log10 likelihood ratio test statistic is calculated. The
#' outputted object is a vector of `nboot` log10 likelihood ratio test
#' statistics from `nboot` distinct bootstrap samples.
#'
#' @param probs founder allele probabilities three-dimensional array for one chromosome only (not a list)
#' @param pheno n by d matrix of phenotypes
#' @param addcovar n by c matrix of additive numeric covariates
#' @param kinship a kinship matrix, not a list
#' @param start_snp positive integer indicating index within probs for start of scan
#' @param n_snp number of (consecutive) markers to use in scan
#' @param pleio_peak_index positive integer index indicating genotype matrix for bootstrap sampling. Typically acquired by using `find_pleio_peak_tib`.
#' @param nboot number of bootstrap samples to acquire and scan
#' @param max_iter maximum number of iterations for EM algorithm
#' @param max_prec stepwise precision for EM algorithm. EM stops once incremental difference in log likelihood is less than max_prec
#' @param cores number of cores to use when calling mclapply to parallelize the bootstrap analysis.
#' @export
#' @importFrom stats var
#' @references Knott SA, Haley CS (2000) Multitrait
#' least squares for quantitative trait loci detection.
#' Genetics 156: 899–911.
#'
#' Walling GA, Visscher PM, Haley CS (1998) A comparison of
#' bootstrap methods to construct confidence intervals in QTL mapping.
#' Genet. Res. 71: 171–180.
#' @examples
#'
#' n <- 50
#' pheno <- matrix(rnorm(2 * n), ncol = 2)
#' rownames(pheno) <- paste0("s", 1:n)
#' colnames(pheno) <- paste0("tr", 1:2)
#' probs <- array(dim = c(n, 2, 5))
#' probs[ , 1, ] <- rbinom(n * 5, size = 1, prob = 0.2)
#' probs[ , 2, ] <- 1 - probs[ , 1, ]
#' rownames(probs) <- paste0("s", 1:n)
#' colnames(probs) <- LETTERS[1:2]
#' dimnames(probs)[[3]] <- paste0("m", 1:5)
#' boot_pvl(probs = probs, pheno = pheno,
#'         start_snp = 1, n_snp = 5, pleio_peak_index = 3, nboot = 1, cores = 1)
#'
#'
#' @return numeric vector of (log) likelihood ratio test statistics from `nboot_per_job` bootstrap samples
#'
boot_pvl <- function(probs,
                     pheno,
                     addcovar = NULL,
                     kinship = NULL,
                     start_snp = 1,
                     n_snp,
                     pleio_peak_index,
                     nboot = 1,
                     max_iter = 1e+04,
                     max_prec = 1 / 1e+08,
                     cores = 1
                     )
    {

    inputs <- process_inputs(probs = probs,
                             pheno = pheno,
                             addcovar = addcovar,
                             kinship = kinship,
                             max_iter = max_iter,
                             max_prec = max_prec)
    ## define X1 - a single marker's allele probabilities
    X1 <- inputs$probs[ , , pleio_peak_index]
    if (!is.null(inputs$addcovar)) {
        Xpre <- cbind(X1, inputs$addcovar)
    } else {
        Xpre <- X1
    }
    d_size <- ncol(inputs$pheno)
    Xlist <- vector(length = d_size)
    Xlist <- lapply(Xlist, FUN = function(x){x <- Xpre; return(x)})
    X <- gemma2::stagger_mats(Xlist)

        Bcol <- rcpp_calc_Bhat2(X = X,
                         Sigma_inv = inputs$Sigma_inv,
                         Y = as.vector(as.matrix(inputs$pheno))
    )
    B <- matrix(data = Bcol, nrow = ncol(Xpre), ncol = d_size, byrow = FALSE)
    # Start loop to get Ysim matrices
    Ysimlist <- list()
    for (i in 1:nboot) {
        foo <- sim1(X = X, B = B, Sigma = inputs$Sigma)
        Ysim <- matrix(foo, ncol = d_size, byrow = FALSE)
        rownames(Ysim) <- rownames(inputs$pheno)
        colnames(Ysim) <- paste0("t", 1:d_size)
        Ysimlist[[i]] <- Ysim
    }
    # prepare table of marker indices for each call of scan_pvl_clean
    mytab <- prep_mytab(d_size = d_size, n_snp = n_snp)

    scan_out <- parallel::mclapply(X = Ysimlist,
                                   FUN = scan_pvl_clean,
                                   mc.cores = cores,
                                   probs = inputs$probs,
                                   addcovar = inputs$addcovar,
                                   Sigma_inv = inputs$Sigma_inv,
                                   Sigma = inputs$Sigma,
                                   start_snp = start_snp,
                                   mytab = mytab,
                                   n_snp = n_snp
                                   )
    lrt <- parallel::mclapply(X = scan_out, FUN = function(x){
        x %>%
            calc_profile_lods() %>%
            dplyr::select(profile_lod) %>%
            max()
    })
    return(unlist(lrt))
}

library(qtl2pleio)
library(testthat)
context("testing boot_pvl")

iron <- qtl2::read_cross2(system.file("extdata", "iron.zip", package="qtl2"))
# insert pseudomarkers into map
map <- qtl2::insert_pseudomarkers(iron$gmap, step=1)
# calculate genotype probabilities
probs <- qtl2::calc_genoprob(iron, map, error_prob=0.002)
# grab phenotypes and covariates; ensure that covariates have names attribute
pheno <- iron$pheno
# leave-one-chromosome-out kinship matrices
kinship <- qtl2::calc_kinship(probs, "loco")$`1`
# get founder allele probabilites
probs <- qtl2::genoprob_to_alleleprob(probs)$`1`
ac <- matrix(as.numeric(iron$covar$sex == "m", ncol = 1))
colnames(ac) <- "sex"
rownames(ac) <- rownames(probs)

set.seed(2018-10-22)
nb <- 2
test_that("output is vector of length nboot", {
  expect_length(boot_pvl(probs = probs,
                         pheno = pheno,
                         kinship = kinship,
                         start_snp = 1,
                         n_snp = 10,
                         pleio_peak_index = 5,
                         nboot = nb, cores = 1),
                nb
                )

})

#' Calculate Vg and Ve from d-variate phenotype and kinship
#'
#' @param pheno n by d matrix of phenotypes
#' @param kinship a kinship matrix, n by n
#' @param X1pre n by c design matrix. c = 1 to ignore genotypes
#' @param max_iter maximum number of EM iterations
#' @param max_prec maximum precision for stepwise increments in EM algorithm
#' @param covariates a n by n.cov matrix of numeric covariates
#' @return a list with 2 named components, Vg and Ve. Each is a d by d covariance matrix.
#' @examples
#' calc_covs(pheno = matrix(data = rnorm(100), nrow = 50, ncol = 2), kinship = diag(50))
#' @export
calc_covs <- function(pheno, kinship, X1pre = rep(1, nrow(kinship)), max_iter = 1e+06, max_prec = 1/1e+08,
    covariates = NULL) {
    e_out <- gemma2::eigen2(kinship)
    U <- e_out$vectors
    eval <- e_out$values
    n_mouse <- nrow(kinship)
    if (is.null(covariates)) {
        X1 <- t(X1pre) %*% U
    } else {
        X1 <- t(cbind(X1pre, covariates)) %*% U
    }
    Y <- t(pheno) %*% U
    d <- ncol(pheno)
    # run MphEM with only a design matrix that contains only the intercept term & covariates, if any(and
    # not any genotype info)
    foo <- gemma2::MphEM(max_iter = max_iter, max_prec = max_prec, X = X1, Y = Y, eval = eval, V_g = diag(d),
        V_e = diag(d))
    Vg <- foo[[length(foo)]][[2]]
    Ve <- foo[[length(foo)]][[3]]
    out <- list(Vg = Vg, Ve = Ve)
    return(out)
}

library(qtl2pleio)

context("calc_covs testing")

phe <- matrix(runif(200), ncol = 2)
sex <- matrix(rbinom(100, size = 1, prob = 1 / 2), ncol = 1)

calc_covs(phe, kinship = diag(100)) -> cc1
calc_covs(phe, kinship = diag(100), covariates = sex) -> cc2

phe3 <- matrix(runif(300), ncol = 3)
phe5 <- matrix(runif(500), ncol = 5)


test_that("calc_covs returns different answers when covariates are used vs. not used", {
  expect_false(identical(cc1$Vg, cc2$Vg))
  expect_false(identical(cc1$Ve, cc2$Ve))
})

test_that("calc_covs, when kinship is identity, has output that sums to give same result as cov()", {
  expect_equal(round(cc1$Vg + cc1$Ve, 4), round(cov(phe), 4), )
})

test_that("calc_covs accommodates d-variate phenotype for d more than 2 & Vg = Ve when kinship is identity", {
  expect_equal(calc_covs(phe3, diag(100))$Vg, calc_covs(phe3, diag(100))$Ve)
  expect_equal(calc_covs(phe5, diag(100))$Vg, calc_covs(phe5, diag(100))$Ve)
  expect_equal(calc_covs(phe5, diag(100), covariates = sex)$Vg, calc_covs(phe5, diag(100), covariates = sex)$Ve)
})

test_that("calc_covs outputs are full rank matrices", {
  expect_equal(qr(calc_covs(phe3, diag(100))$Vg)$rank, 3)
  expect_equal(qr(calc_covs(phe3, diag(100))$Ve)$rank, 3)
  expect_equal(qr(calc_covs(phe5, diag(100))$Ve)$rank, 5)
  expect_equal(qr(calc_covs(phe5, diag(100))$Vg)$rank, 5)
  expect_equal(qr(calc_covs(phe3, diag(100), covariates = sex)$Vg)$rank, 3)
  expect_equal(qr(calc_covs(phe3, diag(100), covariates = sex)$Ve)$rank, 3)
  expect_equal(qr(calc_covs(phe5, diag(100), covariates = sex)$Ve)$rank, 5)
  expect_equal(qr(calc_covs(phe5, diag(100), covariates = sex)$Vg)$rank, 5)
})

test_that("calc_covs outputs are symmetric", {
  expect_true(isSymmetric(calc_covs(phe3, diag(100))$Vg, tol = 0.001))
  expect_true(isSymmetric(calc_covs(phe3, diag(100))$Ve, tol = 0.001))
  expect_true(isSymmetric(calc_covs(phe5, diag(100))$Vg, tol = 0.001))
  expect_true(isSymmetric(calc_covs(phe5, diag(100))$Ve, tol = 0.001))
  expect_true(isSymmetric(calc_covs(phe3, diag(100), covariates = sex)$Vg, tol = 0.001))
  expect_true(isSymmetric(calc_covs(phe3, diag(100), covariates = sex)$Ve, tol = 0.001))
  expect_true(isSymmetric(calc_covs(phe5, diag(100), covariates = sex)$Vg, tol = 0.001))
  expect_true(isSymmetric(calc_covs(phe5, diag(100), covariates = sex)$Ve, tol = 0.001))
})

#' Process inputs to scan functions
#'
#' @param probs a three-dimensional array of founder allele probabilities
#' @param pheno a matrix of d trait values
#' @param addcovar a matrix of covariates
#' @param kinship a kinship matrix
#' @param n_snp number of markers
#' @param start_snp index number of start position in the probs object.
#' @param max_iter max number of iterations for EM
#' @param max_prec max precision for stopping EM

process_inputs <- function(probs,
                           pheno,
                           addcovar,
                           kinship,
                           n_snp = dim(probs)[3],
                           start_snp = 1,
                           max_iter = 10 ^ 4,
                           max_prec = 1 / 10 ^ 8){
  if (is.null(probs)) stop("probs is NULL")
  if (is.null(pheno)) stop("pheno is NULL")
  stopifnot(!is.null(rownames(probs)),
            !is.null(colnames(probs)),
            !is.null(dimnames(probs)[[3]]),
            !is.null(rownames(pheno)),
            n_snp > 0,
            start_snp > 0,
            start_snp + n_snp - 1 <= dim(probs)[3]
  )
  # check additional conditions when addcovar is not NULL
  if (!is.null(addcovar)) {
    stopifnot(!is.null(rownames(addcovar)),
              !is.null(colnames(addcovar))
    )
  }
  d_size <- ncol(pheno)  # d_size is the number of univariate phenotypes
  # force things to be matrices
  if(!is.matrix(pheno)) {
    pheno <- as.matrix(pheno)
    if(!is.numeric(pheno)) stop("pheno is not numeric")
  }
  if(is.null(colnames(pheno))){ # force column names
    colnames(pheno) <- paste0("pheno", seq_len(ncol(pheno)))}
  if(!is.null(addcovar)) {
    if(!is.matrix(addcovar)) addcovar <- as.matrix(addcovar)
    if(!is.numeric(addcovar)) stop("addcovar is not numeric")
  }

  # find individuals in common across all arguments
  # and drop individuals with missing covariates or missing *one or more* phenotypes
  # need to consider presence or absence of different inputs: kinship, addcovar
  id2keep <- make_id2keep(probs = probs,
                          pheno = pheno,
                          addcovar = addcovar,
                          kinship = kinship
  )
  # remove - from id2keep vector - subjects with a missing phenotype or covariate
  pheno <- subset_input(input = pheno, id2keep = id2keep)
  subjects_phe <- check_missingness(pheno)
  id2keep <- intersect(id2keep, subjects_phe)

  if (!is.null(addcovar)) {
    addcovar <- subset_input(input = addcovar, id2keep = id2keep)
    subjects_cov <- check_missingness(addcovar)
    id2keep <- intersect(id2keep, subjects_cov)
    addcovar <- subset_input(input = addcovar, id2keep = id2keep)
  }
  if (!is.null(addcovar)) {
    addcovar <- drop_depcols(addcovar)
  }
  # Send messages if there are two or fewer subjects
  if (length(id2keep) == 0){stop("no individuals common to all inputs")}
  if (length(id2keep) <= 2){
    stop(paste0("only ", length(id2keep),
                " common individual(s): ",
                paste(id2keep, collapse = ": ")))
  }
  # subset inputs to get all without missingness
  probs <- subset_input(input = probs, id2keep = id2keep)
  pheno <- subset_input(input = pheno, id2keep = id2keep)
  if (d_size != qr(pheno)$rank) stop("Phenotypes matrix is not full rank. Input only full-rank phenotypes matrices.")

  if (!is.null(kinship)) {
    kinship <- subset_kinship(kinship = kinship, id2keep = id2keep)
  }
  if (!is.null(addcovar)) {
    addcovar <- subset_input(input = addcovar, id2keep = id2keep)
  }
  if (!is.null(kinship)){
    # covariance matrix estimation
    message(paste0("starting covariance matrices estimation with data from ", length(id2keep), " subjects."))
    # first, run gemma2::MphEM(), by way of calc_covs(), to get Vg and Ve
    cc_out <- calc_covs(pheno, kinship, max_iter = max_iter, max_prec = max_prec, covariates = addcovar)
    Vg <- cc_out$Vg
    Ve <- cc_out$Ve
    message("covariance matrices estimation completed.")
    # define Sigma
    Sigma <- calc_Sigma(Vg, Ve, kinship)
  }
  if (is.null(kinship)){
    # get Sigma for Haley Knott regression without random effect
    Ve <- var(pheno) # get d by d covar matrix
    Sigma <- calc_Sigma(Vg = NULL, Ve = Ve, n_mouse = nrow(pheno))
  }

  # define Sigma_inv
  Sigma_inv <- solve(Sigma)
  # make list to return
  out <- list(probs = probs,
              pheno = pheno,
              addcovar = addcovar,
              kinship = kinship,
              Sigma = Sigma,
              Sigma_inv = Sigma_inv)
  return(out)
}

#' Extract founder allele effects at a single marker from output of qtl2::scan1coef
#'
#' @param marker_index an integer indicating where in the `map` object the peak position (or position of interest) is located
#' @param allele_effects_matrix output of `qtl2::scan1coef` for a single chromosome
#' @param map a map object for the chromosome of interest
#' @param columns which columns to choose within the `allele_effects_matrix`. Default is 1:8 to reflect 8 founder alleles of Diversity Outbred mice
#' @export
#' @return a vector of 8 founder allele effects at a single marker
#' @examples
#' # set up allele effects matrix
#' ae <- matrix(dat = rnorm(100 * 8), ncol = 8, nrow = 100)
#' ae[, 8] <- - rowSums(ae[, 1:7])
#' colnames(ae) <- LETTERS[1:8]
#' rownames(ae) <- paste0(1, "_", 1:100)
#' # set up map
#' map <- 1:100
#' names(map) <- rownames(ae)
#' # call get_effects
#' get_effects(marker_index = 15, allele_effects_matrix = ae, map = map)
#' @return a vector of founder allele effects at a single marker
get_effects <- function(marker_index, allele_effects_matrix, map, columns = 1:8){
  marker_name <- names(map[marker_index])
  return(allele_effects_matrix[rownames(allele_effects_matrix) == marker_name, columns])
}

#' Perform multivariate, one-QTL model fitting for markers on one chromosome
#'
#' `scan_multi_onechr` calculates log likelihood for d-variate phenotype model fits. Inputted parameter `start_snp` indicates where in the `probs` object to start the scan.
#'
#' @param probs an array of founder allele probabilities for a single chromosome
#' @param pheno a matrix of phenotypes
#' @param kinship a kinship matrix for one chromosome
#' @param addcovar a matrix, n subjects by c additive covariates
#' @param start_snp index of where to start the scan within probs
#' @param n_snp the number of (consecutive) markers to include in the scan
#' @param max_iter maximum number of iterations for EM algorithm
#' @param max_prec stepwise precision for EM algorithm. EM stops once incremental difference in log likelihood is less than max_prec
#' @param cores number of cores for parallelization
#' @export
#' @importFrom stats var
#' @references Knott SA, Haley CS (2000) Multitrait
#' least squares for quantitative trait loci detection.
#' Genetics 156: 899–911.
#'
#' Jiang C, Zeng ZB (1995) Multiple trait analysis
#' of genetic mapping for quantitative trait loci.
#' Genetics 140: 1111-1127.
#'
#' Zhou X, Stephens M (2014) Efficient multivariate linear
#' mixed model algorithms for genome-wide association studies.
#' Nature methods 11:407-409.
#'
#' Broman KW, Gatti DM, Simecek P, Furlotte NA, Prins P, Sen S, Yandell BS, Churchill GA (2019)
#' R/qtl2: software for mapping quantitative trait loci with high-dimensional data and
#' multi-parent populations. GENETICS https://www.genetics.org/content/211/2/495.
#'
#'
#' @examples
#' # read data
#' n <- 50
#' pheno <- matrix(rnorm(2 * n), ncol = 2)
#' rownames(pheno) <- paste0("s", 1:n)
#' colnames(pheno) <- paste0("tr", 1:2)
#' probs <- array(dim = c(n, 2, 5))
#' probs[ , 1, ] <- rbinom(n * 5, size = 1, prob = 0.2)
#' probs[ , 2, ] <- 1 - probs[ , 1, ]
#' rownames(probs) <- paste0("s", 1:n)
#' colnames(probs) <- LETTERS[1:2]
#' dimnames(probs)[[3]] <- paste0("m", 1:5)
#' scan_multi_onechr(probs = probs, pheno = pheno, kinship = NULL, cores = 1)
#'
#' @importFrom rlang .data
#' @return a tibble with d + 1 columns. First d columns indicate the genetic data (by listing the marker ids) used in the design matrix; last is log10 likelihood

scan_multi_onechr <- function(probs,
                     pheno,
                     kinship = NULL,
                     addcovar = NULL,
                     start_snp = 1,
                     n_snp = dim(probs)[3],
                     max_iter = 1e+04,
                     max_prec = 1 / 1e+08,
                     cores = 1
)
{
  inputs <- process_inputs(probs = probs, pheno = pheno, kinship = kinship, addcovar = addcovar, max_iter = max_iter, max_prec = max_prec)
  # prepare table of marker indices for each call of scan_pvl_clean
  d_size <- ncol(inputs$pheno)
  mytab <- prep_mytab(d_size = d_size, n_snp = n_snp, pvl = FALSE)
  # set up parallel analysis
  out <- scan_pvl_clean(mytab = mytab,
                        addcovar = inputs$addcovar,
                        probs = inputs$probs,
                        Sigma_inv = inputs$Sigma_inv,
                        Sigma = inputs$Sigma,
                        start_snp = start_snp,
                        pheno = inputs$pheno,
                        n_snp = n_snp, cores = cores
  )
  return(out)
}

#' Perform multivariate, one-QTL model fitting for markers on all chromosomes
#'
#' The function first discards individuals with one or more missing phenotypes or missing covariates.
#' It then infers variance components, Vg and Ve. Both Vg and Ve
#' are d by d covariance matrices. It uses an expectation maximization algorithm, as
#' implemented in the `gemma2` R package. `gemma2` R package is an R implementation of the
#' GEMMA algorithm for multivariate variance component estimation (Zhou & Stephens 2014 Nature methods).
#' Note that variance components are fitted on a model that uses the d-variate phenotype
#' but contains no genetic information. This model does, however,
#' use the specified covariates (after dropping dependent columns
#' in the covariates matrix).
#' These inferred covariance matrices, \eqn{\hat{Vg}} and \eqn{\hat{Ve}},
#' are then used in subsequent model fitting via
#' generalized least squares.
#' Generalized least squares model fitting is applied to every marker on
#' every chromosome.
#' For a single marker, we fit the model:
#' \deqn{vec(Y) = Xvec(B) + vec(G) + vec(E)} where
#' \deqn{G \sim MN(0, K, \hat{Vg})} and \deqn{E \sim MN(0, I, \hat{Ve})} where \eqn{MN} denotes the matrix-variate
#' normal distribution with three parameters: mean matrix, covariance among rows, and
#' covariance among columns. \eqn{vec} denotes the vectorization operation, ie, stacking by columns.
#' \eqn{K} is a kinship matrix, typically calculated by leave-one-chromosome-out methods.
#' \eqn{Y} is the n by d phenotypes matrix. \eqn{X} is a block-diagonal nd by fd matrix consisting of
#' d blocks each of dimension n by f. Each n by f block (on the diagonal) contains a matrix of
#' founder allele probabilities for the n subjects at a single marker. The off-diagonal blocks
#' have only zero entries.
#' The log-likelihood is returned for each model. The outputted object is a tibble with
#' d + 1 columns. The first d columns specify the markers used in the corresponding model fit, while
#' the last column specifies the log-likelihood value at that d-tuple of markers.

#' @param probs_list an list of arrays of founder allele probabilities
#' @param pheno a matrix of phenotypes
#' @param kinship_list a list of kinship matrices, one for each chromosome
#' @param addcovar a matrix, n subjects by c additive covariates
#' @param cores number of cores for parallelization via parallel::mclapply()
#' @export
#' @importFrom stats var
#' @references Knott SA, Haley CS (2000) Multitrait
#' least squares for quantitative trait loci detection.
#' Genetics 156: 899–911.
#'
#' Jiang C, Zeng ZB (1995) Multiple trait analysis
#' of genetic mapping for quantitative trait loci.
#' Genetics 140: 1111-1127.
#'
#' Zhou X, Stephens M (2014) Efficient multivariate linear
#' mixed model algorithms for genome-wide association studies.
#' Nature methods 11:407-409.
#'
#' Broman KW, Gatti DM, Simecek P, Furlotte NA, Prins P, Sen S, Yandell BS, Churchill GA (2019)
#' R/qtl2: software for mapping quantitative trait loci with high-dimensional data and
#' multi-parent populations. GENETICS https://www.genetics.org/content/211/2/495.
#'
#'
#' @examples
#' # read data
#' n <- 50
#' pheno <- matrix(rnorm(2 * n), ncol = 2)
#' rownames(pheno) <- paste0("s", 1:n)
#' colnames(pheno) <- paste0("tr", 1:2)
#' probs <- array(dim = c(n, 2, 5))
#' probs[ , 1, ] <- rbinom(n * 5, size = 1, prob = 0.2)
#' probs[ , 2, ] <- 1 - probs[ , 1, ]
#' rownames(probs) <- paste0("s", 1:n)
#' colnames(probs) <- LETTERS[1:2]
#' dimnames(probs)[[3]] <- paste0("m", 1:5)
#' scan_multi_oneqtl(probs_list = list(probs, probs), pheno = pheno, cores = 1)
#'
#' @importFrom rlang .data
#' @return a tibble with d + 1 columns. First d columns indicate the genetic data (by listing the marker ids) used in the design matrix; last is log10 likelihood

scan_multi_oneqtl <- function(probs_list,
                              pheno,
                              kinship_list = NULL,
                              addcovar = NULL,
                              cores = 1
                              ){
  if (is.null(kinship_list)) {out_list <- parallel::mclapply(X = probs_list,
                                 mc.cores = cores,
                                 FUN = function(x){
                                                  scan_multi_onechr(probs = x,
                                                                    pheno = pheno,
                                                                    kinship = NULL,
                                                                    addcovar = addcovar, cores = cores
                                                  )
                                 })} else {
                                   out_list <- parallel::mclapply(X = probs_list,
                                                                  mc.cores = cores,
                                                                 FUN = function(x){
                                                                   scan_multi_onechr(probs = x,
                                                                                     pheno = pheno,
                                                                                     kinship = kinship_list,
                                                                                     addcovar = addcovar, cores = cores
                                                                   )
                                                                 })
                                                }
  # calculate log10lik for "null" model without genotypes
  if (!is.null(kinship_list)){
    inputs <- process_inputs(probs = probs_list[[1]], # arbitrary choice of which array
                           pheno = pheno,
                           addcovar = addcovar,
                           kinship = kinship_list[[1]] #arbitrary choice of which kin matrix
                           )
  } else {
    inputs <- process_inputs(probs = probs_list[[1]], # arbitrary choice of which array
                             pheno = pheno,
                             addcovar = addcovar,
                             kinship = kinship_list #arbitrary choice of which kin matrix
    )
  }
  n <- nrow(inputs$pheno)
  d_size <- ncol(inputs$pheno)
  if (is.null(addcovar)) {
    Xlist <- lapply(X = as.list(1:d_size),
                  FUN = function(x){return(matrix(data = rep(1, n),
                                                               nrow = n,
                                                               ncol = 1))})
  } else {
    Xlist <- lapply(X = as.list(1:d_size),
                          FUN = function(x){return(cbind(inputs$addcovar, matrix(data = rep(1, n),
                                                          nrow = n,
                                                          ncol = 1)))})
  }
  X <- gemma2::stagger_mats(Xlist)
  Bhat <- rcpp_calc_Bhat2(X = X,
                          Sigma_inv = inputs$Sigma_inv,
                          Y = as.vector(as.matrix(inputs$pheno))
  )
  mymu <- as.vector(X %*% Bhat)
  out <- rcpp_log_dmvnorm2(inv_S = inputs$Sigma_inv,
                           mu = mymu,
                           x = as.vector(as.matrix(inputs$pheno)),
                           S = inputs$Sigma
                           ) / log(10)
  out_list <- parallel::mclapply(X = out_list, mc.cores = cores,
                    FUN = function(x){
                      x %>%
                        dplyr::mutate(null_log10lik = as.numeric(out))

  })

  return(out_list)
  }

#' Permute the phenotypes matrix and then scan the genome. Record the genomewide greatest LOD score for each permuted data set.
#'
#' @param probs_list a list of founder allele probabilities, one array per chromosome
#' @param pheno a matrix of trait values
#' @param kinship_list a list of kinship matrices, one per chromosome
#' @param addcovar a matrix of covariate values
#' @param n_perm positive integer for the number of permuted data sets to scan.
#' @param cores number of cores for parallelization
#' @return a vector of `n_perm` max lod statistics
#' @export

scan_multi_oneqtl_perm <- function(probs_list,
                              pheno,
                              kinship_list = NULL,
                              addcovar = NULL,
                              n_perm = 1,
                              cores = 1
){
  # create a permuted phenotypes matrix
  n <- nrow(pheno)
  phe <- list() # list of permuted data sets
  for (i in 1:n_perm){
    permuted_pheno <- pheno[sample(1:n, replace = FALSE), , drop = FALSE]
    rownames(permuted_pheno) <- rownames(pheno)
    phe[[i]] <- permuted_pheno
  }
  parallel::mclapply(X = phe, mc.cores = cores, FUN = function(x){
    sout <- scan_multi_oneqtl(probs_list = probs_list,
                      pheno = x,
                      kinship_list = kinship_list,
                      addcovar = addcovar
                      )
    maxlod_perchr <- parallel::mclapply(X = sout, mc.cores = cores, FUN = function(x){
      x %>%
        dplyr::select(lod) %>%
        max()
    })
    maxlod <- max(maxlod_perchr)
    return(maxlod)
  })
}

#' Simulate a single multivariate data set consisting of n subjects and d phenotypes for each
#'
#' @param X design matrix (incorporating genotype probabilities from two loci), dn by df
#' @param B a matrix of allele effects, f rows by d columns
#' @param Sigma dn by dn covariance matrix
#' @export
#' @return a vector of length dn. The first n entries are for trait 1, the second n for trait 2, etc.
#' @examples
#' n_mouse <- 20
#' geno <- rbinom(n = n_mouse, size = 1, prob = 1 / 2)
#' X <- gemma2::stagger_mats(geno, geno)
#' B <- matrix(c(1, 2), ncol = 2, nrow = 1)
#' sim1(X, B, Sigma = diag(2 * n_mouse))

sim1 <- function(X, B, Sigma) {
    error_vec <- MASS::mvrnorm(n = 1, mu = rep(0, nrow(Sigma)), Sigma = Sigma)
    Y <- X %*% as.vector(B) + error_vec
    return(Y)
}

#' Check whether a vector, x, has all its entries equal to its first entry
#'
#' @param x a vector
#' @export
#' @return a logical indicating whether all vector entries are the same
#' @examples
#' x <- 1:5
#' check_identical(x)
#' y <- rep(1, 5)
#' check_identical(y)

check_identical <- function(x) {
    stopifnot(is.vector(x))
    n <- length(x)
    return(identical(x, rep(x[1], n)))
}

#' Subset an input object - allele probabilities array or phenotypes matrix or covariates matrix. Kinship has its own subset function
#'
#' An inputted matrix or 3-dimensional array is first subsetted - by rownames - to remove
#' those subjects who are not in `id2keep`. After that, the object's rows
#' are ordered to match the ordering of subject ids in the vector `id2keep`. This
#' (possibly reordered) object is returned.
#'
#' @param input a matrix of either phenotypes or covariates or array of allele probabilities
#' @param id2keep a character vector of subject ids to identify those subjects that are shared by all inputs
#' @export
#' @examples
#' # define s_id
#' s_id <- paste0("s", 1:10)
#' # set up input matrix
#' foo <- matrix(data = rnorm(10 * 3), nrow = 10, ncol = 3)
#' rownames(foo) <- s_id
#' subset_input(input = foo, id2keep = s_id)
#' @return an object resulting from subsetting of `input`. Its rows are ordered per `id2keep`
#'
subset_input <- function(input, id2keep) {
    if (is.null(dim(input))){
        stop("input must have dim = 2 or 3")
    }
    if (length(dim(input)) == 2) {
        input <- input[rownames(input) %in% id2keep, , drop = FALSE]
        out <- input[match(rownames(input), id2keep), , drop = FALSE]
    }
   if (length(dim(input)) == 3) {
       input <- input[rownames(input) %in% id2keep, , , drop = FALSE]
       out <- input[match(rownames(input), id2keep), , , drop = FALSE]
    }
    return(out)
}

#' Subset a kinship matrix to include only those subjects present in all inputs
#'
#' Since a kinship matrix has subject ids in both rownames and colnames, so we need to
#' remove rows and columns according to names in `id2keep`. We first remove rows and
#' columns of subjects that are not in `id2keep`. We then order rows and columns of the
#' resulting matrix by the ordering in `id2keep`.
#'
#' @param kinship a kinship matrix
#' @param id2keep a character vector of subject ids to identify those subjects that are shared by all inputs
#' @export
subset_kinship <- function(kinship, id2keep) {
    if (is.null(dim(kinship))){
        stop("kinship matrix must have dim = 2")
    }
    kinship <- kinship[rownames(kinship) %in% id2keep, colnames(kinship) %in% id2keep]
    return(kinship[match(rownames(kinship), id2keep), match(colnames(kinship), id2keep)])
}

#' Identify shared subject ids among all inputs: covariates, allele probabilities array, kinship, and phenotypes
#'
#' We consider only those inputs that are not NULL. We then use `intersect` on pairs
#' of inputs' rownames to identify those subjects are shared among all non-NULL inputs.
#'
#' @param probs an allele probabilities array
#' @param pheno a phenotypes matrix
#' @param addcovar a covariates matrix
#' @param kinship a kinship matrix
#' @export
#' @return a character vector of subject IDs common to all (non-null) inputs
make_id2keep <- function(probs,
                         pheno,
                         addcovar = NULL,
                         kinship = NULL) {
    if (is.null(addcovar) & !is.null(kinship)) {
        shared <- intersect(rownames(probs), rownames(pheno))
        id2keep <- intersect(shared, rownames(kinship))
    }
    if (!is.null(addcovar) & !is.null(kinship)){
        shared1 <- intersect(rownames(probs), rownames(pheno))
        shared2 <- intersect(rownames(kinship), rownames(addcovar))
        id2keep <- intersect(shared1, shared2)
    }
    if (is.null(addcovar) & is.null(kinship)) {
        id2keep <- intersect(rownames(probs), rownames(pheno))
    }
    if (!is.null(addcovar) & is.null(kinship)){
        shared1 <- intersect(rownames(probs), rownames(pheno))
        id2keep <- intersect(shared1, rownames(addcovar))
    }
    return(id2keep)
}

#' Check for missingness in phenotypes or covariates
#'
#' We use `is.finite` from base R to identify those subjects that have one or
#' more missing values in `input_matrix`. We then return a character vector of subjects
#' that have no missingness in `input_matrix`.
#'
#' @param input_matrix phenotypes or covariates matrix
#' @export
#' @return character vector of subjects that have no missingness
check_missingness <- function(input_matrix){
    nr <- nrow(input_matrix)
    nc <- ncol(input_matrix)
    foo <- is.finite(input_matrix)
    bar <- matrix(data = foo, ncol = nc, nrow = nr)
    indic <- as.logical(apply(FUN = prod, X = bar, MARGIN = 1))
    return(rownames(input_matrix)[indic])
}


# drop linearly dependent columns
# if intercept=TRUE, add intercept before checking and then remove afterwards
# from Karl Broman's rqtl/qtl2 R package
#' @importFrom stats complete.cases
drop_depcols <-
    function(covar=NULL, add_intercept=FALSE, tol=1e-12)
    {
        if(is.null(covar)) return(covar)

        if(!is.matrix(covar)) covar <- as.matrix(covar)
        if(add_intercept) covar <- cbind(rep(1, nrow(covar)), covar)

        if(ncol(covar) <= 1) return(covar)

        X <- covar[complete.cases(covar),,drop=FALSE]

        # deal with NAs by omitting those rows before
        indep_cols <- sort(find_lin_indep_cols(X, tol))

        if(add_intercept) {

            target_ncol <- length(indep_cols)

            n_it <- 0
            while(!(1 %in% indep_cols)) {
                # don't want to omit the first column (the intercept)
                # need to work harder...
                #  - drop one column at a time other the intercept
                #  - when you find a column that doesn't reduce the target number of columns, drop it
                #  - check again if the intercept is being retained; if not, repeat

                for(i in seq_len(ncol(X))[-1]) { # loop over all but the first column (the intercept)
                    indep_cols <- find_lin_indep_cols(X[,-i,drop=FALSE], tol)
                    if(length(indep_cols) == target_ncol) { # this one definitely dependent; omit it
                        X <- X[,-i,drop=FALSE]
                        break
                    }
                }

                # new set of linearly independent columns
                indep_cols <- sort(find_lin_indep_cols(X, tol))

                n_it <- n_it+1 # count number of iterations and bail if it's large
                if(n_it > ncol(covar)+5) { # something is seriously messed up
                    stop("Not able to find set of linearly independent columns")
                }
            }

            # now drop the intercept
            indep_cols <- indep_cols[-1]
        }
        if(length(indep_cols)==0) return(NULL)

        covar[, indep_cols, drop=FALSE]
    }

library(qtl2pleio)
library(testthat)
context("testing correct subsetting of inputs: phenotypes, allele probabilities, covariates, and kinship")

# setup
## define subject ids
s_id <- paste0("s", 101:200)
## define probs
probs_pre <- rbinom(n = 100 * 10, size = 1, prob = 1 / 2)
probs <- array(data = probs_pre, dim = c(100, 1, 10))
dimnames(probs)[[3]] <- paste0("Marker", 1:10)
colnames(probs) <- "A"
rownames(probs) <- s_id
# define Y
Y_pre <- runif(200)
Y <- matrix(data = Y_pre, nrow = 100)
colnames(Y) <- c("y1", "y2")
rownames(Y) <- s_id
# define covariates
covariates <- matrix(c(runif(99), NA), nrow = 100, ncol = 1)
colnames(covariates) <- "c1"
rownames(covariates) <- s_id
cov2 <- matrix(c(covariates[1:99], 10), nrow = 100, ncol = 1)
colnames(cov2) <- "c1"
rownames(cov2) <- s_id

Y2 <- Y
Y2[1, 2] <- NA
# define kinship
K1 <- diag(100)
rownames(K1) <- s_id
colnames(K1) <- s_id

test_that("make_id2keep returns correct vector of subject ids both with and without covariates", {
  expect_length(make_id2keep(probs = probs, pheno = Y[1:10, ], addcovar = NULL, kinship = K1), 10)
  expect_length(make_id2keep(probs = probs, pheno = Y[1:10, ], addcovar = covariates, kinship = K1), 10)
})

library(qtl2pleio)
library(testthat)
context("testing correct subsetting - based on common ids - of inputs (with proper
        ordering of subject ids): phenotypes, allele probabilities,
        covariates, and kinship")

# setup
## define subject ids
s_id <- paste0("s", 101:200)
## define probs
probs_pre <- rbinom(n = 100 * 10, size = 1, prob = 1 / 2)
probs <- array(data = probs_pre, dim = c(100, 1, 10))
dimnames(probs)[[3]] <- paste0("Marker", 1:10)
colnames(probs) <- "A"
rownames(probs) <- s_id
# define Y
Y_pre <- runif(200)
Y <- matrix(data = Y_pre, nrow = 100)
colnames(Y) <- c("y1", "y2")
rownames(Y) <- s_id
# define covariates
covariates <- matrix(c(runif(99), NA), nrow = 100, ncol = 1)
colnames(covariates) <- "c1"
rownames(covariates) <- s_id
cov2 <- matrix(c(covariates[1:99], 10), nrow = 100, ncol = 1)
colnames(cov2) <- "c1"
rownames(cov2) <- s_id

Y2 <- Y
Y2[1, 2] <- NA
# define kinship
K1 <- diag(100)
rownames(K1) <- s_id
colnames(K1) <- s_id


# tests

test_that("subset_input, with covariates, when not NULL,
          returns a matrix with the correct number of rows and correct
          subject ids in row names", {
  expect_equal(nrow(subset_input(input = covariates,
                                id2keep = paste0("s", 101:110))),
               10)
  expect_equal(ncol(subset_input(input = covariates,
                                           id2keep = paste0("s", 101:110))),
                         ncol(covariates))

  expect_identical(rownames(subset_input(input = covariates,
                                    id2keep = paste0("s", 101:110))),
                   paste0("s", 101:110))
  expect_identical(rownames(subset_input(input = covariates,
                                         id2keep = paste0("s", 110:101))),
                   paste0("s", 110:101))

})

test_that("subset_input, with pheno, returns
          a matrix with the correct number of
          rows and correct subject ids in row names", {
  expect_equal(nrow(subset_input(input = Y,
                                id2keep = paste0("s", 101:110)
  )), 10)
  expect_equal(ncol(subset_input(input = Y,
                                 id2keep = paste0("s", 101:110)
            )), ncol(Y))
  expect_identical(rownames(subset_input(input = Y,
                                         id2keep = paste0("s", 101:110)
                                         )), paste0("s", 101:110))
  expect_true(is.matrix(subset_input(input = Y,
                                     id2keep = paste0("s", 101:110))))
})

test_that("subset_input, with allele probabilities array,
          returns a matrix with the correct number of rows
          and correct subject ids in row names and that
          dimension = 3 is preserved", {
  expect_equal(nrow(subset_input(input = probs,
                                id2keep = paste0("s", 101:110)
                                )), 10)
  expect_identical(rownames(subset_input(input = probs,
                                         id2keep = paste0("s", 101:110)
                                         )), paste0("s", 101:110))
  expect_equal(length(dim(subset_input(input = probs,
                                         id2keep = paste0("s", 101:110)
                                       ))), 3)
})

test_that("subset_kinship returns a matrix with the correct number of
          rows & columns and correct subject ids in row names and column names", {
  expect_equal(dim(subset_kinship(kinship = K1,
                                  id2keep = paste0("s", 101:110)))[1],
               10)
  expect_equal(dim(subset_kinship(kinship = K1,
                                  id2keep = paste0("s", 101:110))
                   )[2],
               10)
  expect_identical(rownames(subset_kinship(kinship = K1,
                                         id2keep = paste0("s", 101:110))),
                   paste0("s", 101:110))
  expect_identical(colnames(subset_kinship(kinship = K1,
                                         id2keep = paste0("s", 101:110)
  )), paste0("s", 101:110))
  expect_identical(colnames(subset_kinship(kinship = K1,
                                           id2keep = paste0("s", 110:101)
  )), paste0("s", 110:101))
  expect_identical(rownames(subset_kinship(kinship = K1,
                                           id2keep = paste0("s", 110:101)
  )), paste0("s", 110:101))

})

x <- matrix(runif(100), nrow = 10, ncol = 10)
rownames(x) <- paste0("mouse", 1:10)

y <- x[10:1, ]
rownames(y) <- rownames(x)[10:1]
z <- x[10:1, 1, drop = FALSE]
rownames(z)[10] <- "mouse11"

id2keep <- make_id2keep(probs = x, pheno = y)



test_that("subset_inputs arranges subject names in order specified in id2keep", {
  expect_identical(rownames(subset_input(input = x, id2keep = id2keep)),
                   rownames(subset_input(input = y, id2keep = id2keep))
  )
  expect_false(identical(rownames(subset_input(input = x, id2keep = id2keep)),
                         rownames(y)
  )
  )
  expect_identical(x, subset_input(input = y, id2keep = id2keep))
})

## The code here was originally prepared by Karl Broman
## https://github.com/rqtl/qtl2/blob/68883790388f996639f81b1bb2b7c244f5492164/tests/testthat/test-drop_depcols.R
## I edited the qtl2 test file to remove tests for functions that I don't use in qtl2pleio

library(qtl2pleio)

context("drop linearly dependent columns")

test_that("find_lin_indep_cols works", {

  set.seed(20151130)
  x <- cbind(1, sample(0:1, 200, repl=TRUE))

  expect_equal(sort(find_lin_indep_cols(x)), c(1,2))
  expect_equal(sort(find_lin_indep_cols(cbind(x, 1))), c(1,2))
  expect_equal(sort(find_lin_indep_cols(cbind(x, x[,1] + 0.5*x[,2]))), c(2,3))

  X <- matrix(rnorm(1000*10), ncol=10)
  expect_equal(sort(find_lin_indep_cols(X)), 1:10)
  expect_equal(length(find_lin_indep_cols(cbind(rowSums(X), X))), 10)

})

test_that("drop_depcols works", {

  set.seed(20151130)
  n.ind <- 100
  x <- cbind(1,
             sample(0:1, n.ind, replace=TRUE),
             sample(0:1, n.ind, replace=TRUE))

  expect_equal(drop_depcols(NULL), NULL)
  expect_equal(drop_depcols(x), x)
  for(i in 1:ncol(x))
    expect_equal(drop_depcols(x[,i]), x[,i,drop=FALSE])

  X <- cbind(rowSums(x), x)
  expect_equal(ncol(drop_depcols(X)), 3)

})


test_that("Fixed issue 22 re drop_depcols ", {

  # https://github.com/rqtl/qtl2/issues/22
  # ...case where indicators for all categories included

  # create example
  n_per <- 20
  n_group <- 4
  n <- n_per*n_group
  group <- rep(1:n_group, each=n_per)
  covar <- cbind(sex=rep(0:1,n_per*n_group),
                 diet=sample(100:200, n, replace=TRUE))
  for(i in 1:n_group)
    covar <- cbind(covar, (group==i)*1)
  colnames(covar)[-(1:2)] <- paste0("group", 1:n_group)

  # drop dependent columns
  covar_indep <- drop_depcols(covar, TRUE)

  # check that one of the groups got dropped
  expect_true("sex" %in% colnames(covar_indep))
  expect_true("diet" %in% colnames(covar_indep))
  expect_true(ncol(covar_indep) == ncol(covar)-1)

})

#' Calculate a likelihood ratio test statistic from the output of scan_pvl()
#'
#' @param scan_pvl_out outputted tibble from scan_pvl
#' @examples
#' rep(paste0('Marker', 1:3), times = 3) -> marker1
#' rep(paste0('Marker', 1:3), each = 3) -> marker2
#' runif(9, -1, 0) -> ll
#' tibble::tibble(marker1, marker2, ll) -> scan_out
#' calc_lrt_tib(scan_out)
#' @export
#' @return a number, the (log) likelihood ratio test statistic
calc_lrt_tib <- function(scan_pvl_out) {
    nc <- ncol(scan_pvl_out)
    smat <- as.matrix(scan_pvl_out[, -nc])
    # define an indicator vector for which rows to keep when making pleio_tib
    pleio_ind <- apply(FUN = function(x) {
        names(x) <- NULL
        identical(x, rep(x[1], nc - 1))
    }, X = smat, MARGIN = 1)
    pleio_tib <- scan_pvl_out[pleio_ind, ]
    return(max(scan_pvl_out[, nc]) - max(pleio_tib[, nc]))
}

library(qtl2pleio)
library(testthat)
context("testing calc_lrt_tib")

rep(rep(paste0('Marker', 1:2), times = 2), each = 2) -> marker1
rep(rep(paste0('Marker', 1:2), each = 2), each = 2) -> marker2
rep(paste0("Marker", 1:2), times = 4) -> marker3
runif(8, -1, 0) -> ll
tibble::tibble(marker1, marker2, ll) -> foo
bar <- tibble::tibble(marker1, marker2, marker3, ll)



test_that("calc_lrt_tib outputs a single number", {
  expect_true(is.numeric(calc_lrt_tib(foo)))
})

test_that("calc_lrt_tib works with 3-dim scan", {
  expect_true(is.numeric(calc_lrt_tib(bar)))
})

#' qtl2pleio.
#'
#' Testing pleiotropy vs. separate QTL in multiparental populations
#'
#' @useDynLib qtl2pleio, .registration = TRUE
#' @importFrom Rcpp sourceCpp
#' @importFrom stats profile
"_PACKAGE"

.onUnload <- function (libpath) {
  library.dynam.unload("qtl2pleio", libpath)
}

## quiets concerns of R CMD check re: the .'s that appear in pipelines
if(getRversion() >= "2.15.1")  utils::globalVariables(c(".", "lod", "marker_position", "profile_lod", "loglik", "Var1", "profile", "pleio_max", "marker", "trait", "log10lik", "null_log10lik"))

#' Calculate estimated allele effects, B matrix
#'
#' @param X dn by df block-diagonal design matrix that incorporates genetic info for d markers. Note that we can use the same marker data twice.
#' @param Sigma_inv dn by dn inverse covariance matrix, often composed as the inverse of \eqn{K \otimes V_g + I_n \otimes V_e}
#' @param Y dn by 1 matrix, ie, a column vector, of d phenotypes' measurements
#' @return a df by 1 matrix of GLS-estimated allele effects
#' @examples
#' X1 <- as.matrix(rbinom(n = 100, size = 1, prob = 1 / 2))
#' X <- gemma2::stagger_mats(X1, X1)
#' Sigma_inv <- diag(200)
#' Y <- runif(200)
#' calc_Bhat(X, Sigma_inv, Y)
#' @export
calc_Bhat <- function(X, Sigma_inv, Y) {
    out <- solve(t(X) %*% Sigma_inv %*% X) %*% t(X) %*% Sigma_inv %*% as.vector(as.matrix(Y))
    return(out)
}

library(qtl2pleio)
context("Check calc_Bhat")

# setup
X1 <- as.matrix(rbinom(n = 100, size = 1, prob = 1 / 2))
X <- gemma2::stagger_mats(X1, X1)
Sigma_inv <- matrix(data = rep(1, 200 * 200), nrow = 200)
Y <- runif(200)


test_that("error is returned when inputted Sigma_inv is not positive definite", {
  expect_error(calc_Bhat(X = X, Sigma_inv = Sigma_inv, Y = Y))
})
test_that("dimensions of returned object are correct", {
  expect_equal(nrow(calc_Bhat(X = X, Sigma_inv = diag(200), Y = Y)), 2)
  expect_equal(ncol(calc_Bhat(X = X, Sigma_inv = diag(200), Y = Y)), 1)
})

#' Calculate matrix square root for a covariance matrix
#'
#' @param A covariance matrix
#'
calc_sqrt_mat <- function(A) {
    rcppeigen_sqrt(A)
}


#' Calculate matrix inverse square root for a covariance matrix
#'
#' @param A covariance matrix
#'
calc_invsqrt_mat <- function(A) {
    rcppeigen_invsqrt(A)
}

#' Add physical map contents to tibble
#'
#' @family profile lod tibble functions
#' @param tib a tibble with 3 columns: marker, trace, and profile lod values, typically outputted by calc_profile_lods()
#' @param pmap a physical map for a single chromosome
#' @return a tibble with 4 columns: marker, trait, profile_lod, marker_position
#' @examples
#' pm <- 1:3
#' names(pm) <- as.character(paste0('m', 1:3))
#' expand.grid(paste0('m', 1:3), paste0('m', 1:3)) %>%
#'     tibble::as_tibble() %>%
#'     dplyr::mutate(log10lik = rgamma(9, 5)) %>%
#'     calc_profile_lods() %>%
#'     add_pmap(pm)
#' @export
#' @importFrom rlang .data

add_pmap <- function(tib, pmap) {
    tibble::tibble(marker = as.character(names(pmap)), marker_position = pmap) %>%
        dplyr::inner_join(tib)
}





#' Calculate profile lods for all traits
#'
#' @param scan_pvl_out tibble outputted from scan_pvl
#' @export
#' @return a tibble with 3 columns, indicating 'marker identity, trace (pleiotropy or profile1, profile2, etc.), and value of the profile lod (base 10) for that trace at that marker.

calc_profile_lods <- function(scan_pvl_out){
    nc <- ncol(scan_pvl_out)
    out <- list()
    for (i in 1:(nc - 1)) {
        out[[i]] <- scan_pvl_out %>%
            dplyr::group_by_at(i) %>%
            dplyr::summarise(profile = max(log10lik)) %>%
            dplyr::mutate(trait = paste0("tr", i)) %>%
            dplyr::rename(marker = paste0("Var", i))
    }
    # Calculate the "pleiotropy" trace
    pleio_trace <- scan_pvl_out %>%
        dplyr::filter_all(dplyr::all_vars(. == Var1 | is.numeric(.))) %>%
        dplyr::rename(profile = log10lik) %>%
        dplyr::select(Var1, profile) %>%
        dplyr::rename(marker = Var1) %>%
        dplyr::mutate(trait = "pleiotropy")
    #
    out %>%
        dplyr::bind_rows() %>%
        dplyr::bind_rows(pleio_trace) %>%
        dplyr::mutate(pleio_max = max(pleio_trace$profile)) %>%
        dplyr::mutate(profile_lod = profile - pleio_max) %>%
        dplyr::select(marker, trait, profile_lod)
}

#' Calculate log likelihood for a multivariate normal
#'
#' @param inv_S inverse covariance matrix
#' @param mu mean vector
#' @param x data vector
#' @param S covariance matrix, ie, the inverse of inv_S
#' @export

rcpp_log_dmvnorm2 <- function(inv_S, mu, x, S) {
    stopifnot(dim(inv_S) == dim(S),
              nrow(inv_S) == ncol(inv_S),
              nrow(inv_S) == length(mu),
              length(mu) == length(x))
    n <- length(x)
    diag <- rcppeigen_get_diag(S)
    return((-n / 2) * log(2 * pi) - sum(log(diag)) - (1 / 2) * (x - mu) %*% inv_S %*% (x - mu))
}

library(qtl2pleio)
context("testing rcpp_log_dmvnorm2")



test_that("rcpp_log_dmvnorm returns same values as mvtnorm::dmvnorm when log = TRUE", {
  expect_identical(
    as.numeric(rcpp_log_dmvnorm2(inv_S = diag(10), mu = rep(0, 10), x = rep(0, 10), S = diag(10))),
    mvtnorm::dmvnorm(x = rep(0, 10), mean = rep(0, 10), sigma = diag(10), log = TRUE)
  )
})

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

rcppeigen_sqrt <- function(A) {
    .Call(`_qtl2pleio_rcppeigen_sqrt`, A)
}

rcppeigen_invsqrt <- function(A) {
    .Call(`_qtl2pleio_rcppeigen_invsqrt`, A)
}

timesTwo <- function(x) {
    .Call(`_qtl2pleio_timesTwo`, x)
}

find_matching_cols <- function(mat, tol = 1e-12) {
    .Call(`_qtl2pleio_find_matching_cols`, mat, tol)
}

find_lin_indep_cols <- function(mat, tol = 1e-12) {
    .Call(`_qtl2pleio_find_lin_indep_cols`, mat, tol)
}

formX_intcovar <- function(probs, addcovar, intcovar, position, has_intercept = TRUE) {
    .Call(`_qtl2pleio_formX_intcovar`, probs, addcovar, intcovar, position, has_intercept)
}

expand_genoprobs_intcovar <- function(probs, intcovar) {
    .Call(`_qtl2pleio_expand_genoprobs_intcovar`, probs, intcovar)
}

weighted_matrix <- function(mat, weights) {
    .Call(`_qtl2pleio_weighted_matrix`, mat, weights)
}

weighted_3darray <- function(array, weights) {
    .Call(`_qtl2pleio_weighted_3darray`, array, weights)
}

matrix_x_matrix <- function(X, Y) {
    .Call(`_qtl2pleio_matrix_x_matrix`, X, Y)
}

matrix_x_vector <- function(X, y) {
    .Call(`_qtl2pleio_matrix_x_vector`, X, y)
}

matrix_x_3darray <- function(X, A) {
    .Call(`_qtl2pleio_matrix_x_3darray`, X, A)
}

rcpp_ols <- function(X, Y) {
    .Call(`_qtl2pleio_rcpp_ols`, X, Y)
}

rcpp_gls <- function(X, Y, Sigma_inv) {
    .Call(`_qtl2pleio_rcpp_gls`, X, Y, Sigma_inv)
}

rcppeigen_invert_matrix <- function(A) {
    .Call(`_qtl2pleio_rcppeigen_invert_matrix`, A)
}

rcppeigen_get_diag <- function(A) {
    .Call(`_qtl2pleio_rcppeigen_get_diag`, A)
}

rcppeigen_get_det <- function(A) {
    .Call(`_qtl2pleio_rcppeigen_get_det`, A)
}

rcppeigen_get_chol <- function(A) {
    .Call(`_qtl2pleio_rcppeigen_get_chol`, A)
}

rcppeigen_get_chol_stable <- function(A) {
    .Call(`_qtl2pleio_rcppeigen_get_chol_stable`, A)
}

rcppeigen_get_chol_diag <- function(A) {
    .Call(`_qtl2pleio_rcppeigen_get_chol_diag`, A)
}

#' Fit a model for a specified d-tuple of markers
#'
#' `fit1_pvl` uses several functions in the package qtl2pleio to fit the
#' linear mixed effects model for a single d-tuple of markers.
#' Creation of `fit1_pvl` - from code that originally resided in `scan_pvl`, enabled
#' parallelization via the `parallel` R package.
#'
#' @param indices a vector of indices for extracting elements of `probs` array
#' @param start_snp an integer to specify the index of the marker where the scan - in call to scan_pvl - starts. This argument is needed because `mytab` has only relative indices (relative to the `start_snp` marker)
#' @param probs founder allele probabilities array
#' @param addcovar additive covariates matrix
#' @param inv_S inverse covariance matrix for the vectorized phenotype
#' @param S covariance matrix for the vectorized phenotype, ie, the inverse of inv_S. By making this a function input, we avoid inverting the matrix many many times.
#' @param pheno a n by d phenotypes matrix
#' @export
#' @return a number, the log-likelihood for the specified model
#' @examples
#' n <- 50
#' pheno <- matrix(rnorm(2 * n), ncol = 2)
#' Vg <- diag(2)
#' Ve <- diag(2)
#' Sigma <- calc_Sigma(Vg, Ve, diag(n))
#' Sigma_inv <- solve(Sigma)
#' probs <- array(dim = c(n, 2, 5))
#' probs[ , 1, ] <- rbinom(n * 5, size = 1, prob = 0.2)
#' probs[ , 2, ] <- 1 - probs[ , 1, ]
#' mytab <- prep_mytab(d_size = 2, n_snp = 5)
#' fit1_pvl(mytab[1, ], start_snp = 1,
#' probs = probs, addcovar = NULL, inv_S = Sigma_inv,
#' S = Sigma,
#' pheno = pheno
#' )

fit1_pvl <- function(indices,
                     start_snp,
                     probs,
                     addcovar,
                     inv_S,
                     S,
                     pheno
                     ){
  indices <- as.numeric(indices)
  if (is.na(indices[length(indices)])) indices <- indices[- length(indices)]
  X_list <- prep_X_list(indices = indices,
                        start_snp = start_snp,
                        probs = probs,
                        covariates = addcovar
  )
  X <- gemma2::stagger_mats(X_list)
  Bhat <- rcpp_calc_Bhat2(X = X,
                          Sigma_inv = inv_S,
                          Y = as.numeric(as.vector(as.matrix(pheno)))
                          )
  mymu <- as.vector(X %*% Bhat)
  out <- rcpp_log_dmvnorm2(inv_S = inv_S,
                           mu = mymu,
                           x = as.numeric(as.vector(as.matrix(pheno))),
                           S = S
                           )
  return(as.numeric(out))
}

#' Estimate allele effects matrix, B hat, with Rcpp functions
#'
#' @param X dn by df block-diagonal design matrix that incorporates genetic info for two markers. Note that we can use the same marker data twice.
#' @param Sigma_inv dn by dn inverse covariance matrix, where its inverse, ie, Sigma, is often composed as \eqn{K \otimes V_g + I_n \otimes V_e}
#' @param Y dn by 1 matrix, ie, a column vector, of d phenotypes' measurements
#' @return a df by 1 matrix of GLS-estimated allele effects
#' @examples
#' X1 <- as.matrix(rbinom(n = 100, size = 1, prob = 1 / 2))
#' X <- gemma2::stagger_mats(X1, X1)
#' Sigma_inv <- diag(200)
#' Y <- runif(200)
#' rcpp_calc_Bhat(X = X, Sigma_inv = Sigma_inv, Y = Y)
#' @export
#'
rcpp_calc_Bhat <- function(X, Sigma_inv, Y) {
    # calc_invsqrt_mat(Sigma) -> invsqrt_Sigma
    invsqrt_Sigma <- calc_sqrt_mat(Sigma_inv)
    Wmat <- invsqrt_Sigma %*% X
    Zmat <- invsqrt_Sigma %*% Y
    out <- rcpp_ols(X = Wmat, Y = Zmat)
    return(out)
}

#' Estimate allele effects matrix, B hat, with Rcpp functions
#'
#' @param X dn by df block-diagonal design matrix that incorporates genetic info for two markers. Note that we can use the same marker data twice.
#' @param Y dn by 1 matrix, ie, a column vector, of d phenotypes' measurements
#' @param Sigma_inv dn by dn inverse covariance matrix, often composed as inverse of \eqn{K \otimes V_g + I_n \otimes V_g}
#' @return a df by 1 matrix of GLS-estimated allele effects
#' @examples
#' X1 <- as.matrix(rbinom(n = 100, size = 1, prob = 1 / 2))
#' X <- gemma2::stagger_mats(X1, X1)
#' Sigma_inv <- diag(200)
#' Y <- runif(200)
#' rcpp_calc_Bhat2(X = X, Y = Y, Sigma_inv = Sigma_inv)
#' @export
#'
rcpp_calc_Bhat2 <- function(X, Y, Sigma_inv) {
    out <- rcpp_gls(X = X, Y = Y, Sigma_inv = Sigma_inv)
    return(out)
}

library(qtl2pleio)
context("testing rcpp_calc_Bhat2 and rcpp_calc_Bhat against calc_Bhat")

# setup
X1 <- as.matrix(as.numeric(rbinom(n = 100, size = 1, prob = 1 / 2))) # NEED! as.numeric instead of integer for Rcpp!
X <- gemma2::stagger_mats(X1, X1)
Sigma_inv <- diag(200)
Y <- runif(200)
# more complicated covariance matrix
Spre <- matrix(data = runif(200 * 200), nrow = 200, ncol = 200)
S <- Spre %*% t(Spre)
solve(S) -> S2_inv


test_that("rcpp_calc_Bhat2 returns same values as rcpp_calc_Bhat and calc_Bhat", {
  expect_equal(
    rcpp_calc_Bhat(X = X , Sigma_inv = Sigma_inv, Y = Y),
    rcpp_calc_Bhat2(X = X, Y = Y, Sigma_inv = Sigma_inv)
  )
  expect_equal(
    rcpp_calc_Bhat(X = X , Sigma_inv = S2_inv, Y = Y),
    rcpp_calc_Bhat2(X = X, Y = Y, Sigma_inv = S2_inv)
  )
  expect_equal(
    calc_Bhat(X = X , Sigma_inv = S2_inv, Y = Y),
    rcpp_calc_Bhat2(X = X, Y = Y, Sigma_inv = S2_inv)
  )
})

#' Plot tidied results of a pvl scan
#'
#' @param dat a profile lod tibble
#' @param units a character vector of length one to indicate units for physical or genetic map
#' @param palette a character vector of length 3 containing strings for colors
#' @param linetype a character vector of length 3 specifying the linetype values for the 3 traces
#' @export
#' @return a ggplot object with profile LODs

plot_pvl <- function(dat, units = "Mb",
                     palette = c("#999999", "#E69F00", "#56B4E9"),
                     linetype = c("solid", "longdash", "dotted"))
{
  dat %>%
    ggplot2::ggplot() +
    ggplot2::geom_line(ggplot2::aes(y = lod, x = marker_position,
                           color = trace, linetype = trace)) +
    ggplot2::scale_colour_manual(values = c("pleiotropy" = palette[1],
                                            "profile1" = palette[2],
                                            "profile2" = palette[3]),
                                 labels = unique(dat$Trace)) +
    ggplot2::scale_linetype_manual(values = c("pleiotropy" = linetype[1],
                                              "profile1" = linetype[2],
                                              "profile2" = linetype[3]),
                                   labels = unique(dat$Trace)) +
    ggplot2::labs(x = paste0("Chromosome position (", units, ")"), y = "LOD")
}

#' Convert `scan_multi_oneqtl` output of `qtl2::scan1` output
#'
#' We convert output of `scan_multi_oneqtl` into format outputted by `qtl2::scan1`.
#'
#' @param sm_output tibble output from scan_multi_oneqtl for one chromosome only
#' @param trait_name character vector (of length one) specifying the trait names
#' @return object of class `scan1`
#' @export
#' @examples
#' # read data
#' iron <- qtl2::read_cross2(system.file("extdata", "iron.zip", package="qtl2"))
#' \dontshow{iron <- iron[,7]}
#'
#' # insert pseudomarkers into map
#' map <- qtl2::insert_pseudomarkers(iron$gmap, step=1)
#'
#' # calculate genotype probabilities
#' probs <- qtl2::calc_genoprob(iron, map, error_prob=0.002)
#'
#' # grab phenotypes and covariates; ensure that covariates have names attribute
#' pheno <- iron$pheno
#' covar <- match(iron$covar$sex, c("f", "m")) # make numeric
#' names(covar) <- rownames(iron$covar)
#' Xcovar <- qtl2::get_x_covar(iron)
#'
#' aprobs <- qtl2::genoprob_to_alleleprob(probs)
#' sm_out <- scan_multi_oneqtl(probs = aprobs, pheno = pheno)
#' sm_to_s1 <- convert_to_scan1_output(sm_out[[1]], trait_name = "tr1and2")
#'
#' # 95% Bayes credible interval for QTL on chr 7, first phenotype
#' qtl2::bayes_int(sm_to_s1, map)
#'
convert_to_scan1_output <- function(sm_output, trait_name){
  lods <- sm_output %>%
    dplyr::mutate(lod = log10lik - null_log10lik) %>%
    dplyr::select(Var1, lod)
  mat <- as.matrix(lods$lod)
  rownames(mat) <- lods$Var1
  colnames(mat) <- trait_name
  class(mat) <- c("scan1", "matrix")
  return(mat)
}

#' Find the marker index corresponding to the peak of the pleiotropy trace in a tibble where the last column contains log likelihood values and the first d columns contain marker ids
#'
#' @param tib a (d+1) column tibble with first d columns containing marker ids and the last containing log likelihood values. Typically this is the output from `scan_pvl`.
#' @param start_snp positive integer, from the two-dimensional scan, that indicates where the scan started on the chromosome
#' @return positive integer indicating marker index for maximum value of log lik under pleiotropy
#' @export
#' @examples
#' marker1 <- rep(paste0('SNP', 1:3), times = 3)
#' marker2 <- rep(paste0('SNP', 1:3), each = 3)
#' loglik <- runif(9, -5, 0)
#' tibble::tibble(marker1, marker2, loglik) -> tib
#' find_pleio_peak_tib(tib, start_snp = 1)

find_pleio_peak_tib <- function(tib, start_snp) {
    nc <- ncol(tib)
    if (nc < 3){stop("tib must have at least 3 columns")}
    if (!is.numeric(unlist(tib[, nc]))){stop("last column of tib must be numeric")}
    smat <- as.matrix(tib[, -nc])
    # define an indicator vector for which rows to keep when making pleio_tib
    pleio_ind <- apply(FUN = function(x) {
        names(x) <- NULL
        identical(x, rep(x[1], nc - 1))
    }, X = smat, MARGIN = 1)
    pleio_tib <- tib[pleio_ind, ]
    ll <- unlist(pleio_tib[, nc])
    return(which.max(ll) + start_snp - 1)
}

library(qtl2pleio)
library(testthat)
context("Testing find_pleio_peak_tib")

marker1 <- rep(paste0('SNP', 1:3), times = 3)
marker2 <- rep(paste0('SNP', 1:3), each = 3)
set.seed(2018-12-25)
loglik <- runif(9, -5, 0)
tibble::tibble(marker1, marker2, loglik) -> tib
tibble::tibble(marker1, marker2, 1:9) -> tib2
tibble::tibble(marker1, marker2, - loglik) -> tib3

test_that("find_pleio_peak_tib returns the index that has the greatest value of loglik",
          {
            expect_equal(which.max(loglik[c(1, 5, 9)]),
                         as.integer(find_pleio_peak_tib(tib, start_snp = 1)))
          })

test_that("Input checks for find_pleio_peak_tib work as expected",
          {
            expect_error(find_pleio_peak_tib(tib2[ , 1:2], start_snp = 1))
            })

---
title: "Using HTCondor for bootstrap analysis"
author: "Frederick Boehm"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Using HTCondor for bootstrap analysis}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---



## Overview

Performing a bootstrap analysis with `qtl2pleio` requires first sampling a (bivariate) phenotype from the inferred bivariate normal distribution, then a two-dimensional, two-QTL QTL scan over the defined scan region. Often, one wants to include more than 100 markers in a scan region. Each two-dimensional scan can take tens of minutes with a single core. Thus, performing a bootstrap analysis with, say, 1000 bootstrap samples is prohibitively time-consuming on most computers.
For this reason, we used a high-throughput computing cluster at the University of Wisconsin-Madison (where `qtl2pleio` was developed). The UW-Madison Center for High-Throughput Computing (CHTC) is available to all UW-Madison researchers. While we recognize that `qtl2pleio` users may not have access to the CHTC, we anticipate that some will have access to computing clusters. Hopefully this vignette can be adapted to your needs.

## Directory structure, files & setup for using Condor

We placed on github a repository that contains the results from our bootstrap analysis of the Recla data.

Here is the repository's url: https://github.com/fboehm/qtl2pleio-manuscript-chtc/

Within this repo are three subdirectories. We'll examine in detail the "Recla-bootstrap" subdirectory.

Within this directory, we created four subdirectories:


1. Rscript
1. data
1. shell_scripts
1. submit_files

The Github repository also contains a fifth subdirectory, `squid` and a sixth subdirectory `results`, that are included for completeness. In practice, files in `squid` were in a distinct directory. We describe it below.

### Files in `Rscript`

The `Rscript` subdirectory contains a file, `boot-Recla-10-22.R` that contains R code for our bootstrap analysis.

```{r}
fn <- file.path("https://raw.githubusercontent.com", 
                "fboehm/qtl2pleio-manuscript/master/chtc", 
                "Recla-bootstrap/Rscript/boot-Recla-10-22.R"
                )
foo <- readLines(fn)
```


Here we display the contents of our `boot-Recla-10-22.R` file.

```{r, echo = FALSE, comment = ""}
cat(foo, sep = "\n")
```




 
### Files in `data`

The data subdirectory contains three rds files, one for each of the three input components.

1. founder allele dosages for Chr 8
2. phenotypes
3. kinship matrix (derived via LOCO method)



### Files in `shell_scripts`

We examine only the file that is needed for the Recla analysis, `boot-Recla-10-22.sh`.

Here is the contents of the file. 

```{bash, eval = FALSE}
#!/bin/bash

# untar your R installation
tar -xzf R13.tar.gz
tar -xzf SLIBS.tar.gz
# make sure the script will use your R installation
export PATH=$(pwd)/R/bin:$PATH
export LD_LIBRARY_PATH=$(pwd)/SS:$LD_LIBRARY_PATH
# run R, with the name of your  R script
R CMD BATCH '--args argname='$1' nsnp='$2' s1='$3' nboot_per_job='$4' run_num='$5'' Rscript/boot-Recla-10-22.R 'boot400_run'$5'-'$1'.Rout'
```

We need to unzip the R installation, which we've named R13.tar.gz, and SLIBS file on the remote computer where the actual computing will occur. Following that, and needed adjustments to the paths, we execute R. Note that we have, in this file, specified the variable values by numbers, like `$1` for `argname`. The values assigned to each of these is specified in the "submit" file.





### Files in `submit_files`

The HTCondor "submit" file provides instructions for controlling the interaction with the high-throughput resources. Below is the text in the submit file `boot-Recla-10-22.sub`

```{bash, eval = FALSE}
# hello-chtc.sub
# My very first HTCondor submit file
s1=650
# start scan at this SNP index value
nsnp=350
# length of scan in number of SNPs
# which run in the experimental design?
run_num=561
#
nboot_per_job=1
#  for almost all jobs), the desired name of the HTCondor log file,
#  and the desired name of the standard error file.  
#  Wherever you see $(Cluster), HTCondor will insert the queue number
#  assigned to this set of jobs at the time of submission.
universe = vanilla
log = boot-$(Process)-run$(run_num).log
error = boot-$(Process)-run$(run_num).err
#
# Specify your executable (single binary or a script that runs several
#  commands), arguments, and a files for HTCondor to store standard
#  output (or "screen output").
#  $(Process) will be a integer number for each job, starting with "0"
#  and increasing for the relevant number of jobs.
executable = ../shell_scripts/boot-Recla-10-22.sh
arguments = $(Process) $(nsnp) $(s1) $(nboot_per_job) $(run_num)
output = boot-$(Process)-run$(run_num).out
#
# Specify that HTCondor should transfer files to and from the
#  computer where each job runs. The last of these lines *would* be
when_to_transfer_output = ON_EXIT
transfer_input_files = ../data,../Rscript,../shell_scripts,http://proxy.chtc.wisc.edu/SQUID/fjboehm/R13.tar.gz,http://proxy.chtc.wisc.edu/SQUID/SLIBS.tar.gz
#
# Tell HTCondor what amount of compute resources
#  each job will need on the computer where it runs.
request_cpus = 1
request_memory = 3GB
request_disk = 1GB
#
requirements = (OpSysMajorVer == 6) || (OpSysMajorVer == 7)

# which computer grids to use:
+WantFlocking = true
+WantGlideIn = true

# Tell HTCondor to run instances of our job:
queue 1000

```

The ordering of the arguments on in the line that starts with `arguments = ` dictates the numbers that are assigned to each argument. These numbers are used in the shell script, above.




### Files on `SQUID`

The UW-Madison CHTC provides for each user disk space on a web proxy. More information about `SQUID` is available online.

In practice, we placed files containing founder allele dosages (for chromosomes of interest) and our (compressed) R installation on `SQUID`.





## Submitting jobs to Condor (at UW-Madison CHTC)

We followed the usual procedure for submitting R jobs with the CHTC. To submit the file `boot-Recla-10-22.sub`, we first changed into its directory. We then typed

```{bash, eval = FALSE}
condor_submit boot-Recla-10-22.sub
```

By using this command from within the `submit_files` subdirectory, we ensure that our outputted files are returned to the `submit_files` subdirectory. We then manually moved the outputted results files to the `results` subdirectory.

We found that up to ten percent of jobs would initially fail and require re-submission. There are a variety of reasons that may explain the need for re-submission. Because of this need, we include in the git repository the file `boot-Recla-10-22-fix.sub`, which gives the code that we used for resubmission. The file `bad-jobs-run561` gives the ids of the jobs that failed the initial runs.



## Session info

```{r}
devtools::session_info()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{subset_kinship}
\alias{subset_kinship}
\title{Subset a kinship matrix to include only those subjects present in all inputs}
\usage{
subset_kinship(kinship, id2keep)
}
\arguments{
\item{kinship}{a kinship matrix}

\item{id2keep}{a character vector of subject ids to identify those subjects that are shared by all inputs}
}
\description{
Since a kinship matrix has subject ids in both rownames and colnames, so we need to
remove rows and columns according to names in `id2keep`. We first remove rows and
columns of subjects that are not in `id2keep`. We then order rows and columns of the
resulting matrix by the ordering in `id2keep`.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{check_identical}
\alias{check_identical}
\title{Check whether a vector, x, has all its entries equal to its first entry}
\usage{
check_identical(x)
}
\arguments{
\item{x}{a vector}
}
\value{
a logical indicating whether all vector entries are the same
}
\description{
Check whether a vector, x, has all its entries equal to its first entry
}
\examples{
x <- 1:5
check_identical(x)
y <- rep(1, 5)
check_identical(y)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calc_Sigma.R
\name{calc_Sigma}
\alias{calc_Sigma}
\title{Calculate the phenotypes covariance matrix Sigma}
\usage{
calc_Sigma(Vg, Ve, kinship = NULL, n_mouse = nrow(kinship))
}
\arguments{
\item{Vg}{d by d genetic covariance matrix for the d phenotypes}

\item{Ve}{d by d error covariance matrix for the d phenotypes}

\item{kinship}{optional n by n kinship matrix. if NULL, Vg is not used.}

\item{n_mouse}{number of subjects}
}
\value{
dn by dn covariance matrix
}
\description{
Calculate the phenotypes covariance matrix Sigma
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prep_mytab.R
\name{prep_X_list}
\alias{prep_X_list}
\title{Create a list of component X matrices for input to stagger_mats, to ultimately create design matrix}
\usage{
prep_X_list(indices, start_snp, probs, covariates)
}
\arguments{
\item{indices}{a vector of integers}

\item{start_snp}{an integer denoting the index (within genotype probabilities array) where the scan should start}

\item{probs}{a three-dimensional array of genotype probabilities for a single chromosome}

\item{covariates}{a matrix of covariates}
}
\value{
a list of design matrices, ultimately useful when constructing the (multi-locus) design matrix
}
\description{
Create a list of component X matrices for input to stagger_mats, to ultimately create design matrix
}
\examples{
pp <- array(rbinom(n = 200, size = 1, prob = 0.5), dim = c(10, 2, 10))
prep_X_list(1:3, 1, probs = pp, covariates = NULL)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calc_profile_lods.R
\name{add_pmap}
\alias{add_pmap}
\title{Add physical map contents to tibble}
\usage{
add_pmap(tib, pmap)
}
\arguments{
\item{tib}{a tibble with 3 columns: marker, trace, and profile lod values, typically outputted by calc_profile_lods()}

\item{pmap}{a physical map for a single chromosome}
}
\value{
a tibble with 4 columns: marker, trait, profile_lod, marker_position
}
\description{
Add physical map contents to tibble
}
\examples{
pm <- 1:3
names(pm) <- as.character(paste0('m', 1:3))
expand.grid(paste0('m', 1:3), paste0('m', 1:3)) \%>\%
    tibble::as_tibble() \%>\%
    dplyr::mutate(log10lik = rgamma(9, 5)) \%>\%
    calc_profile_lods() \%>\%
    add_pmap(pm)
}
\concept{profile lod tibble functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scan_multi_onechr.R
\name{scan_multi_oneqtl}
\alias{scan_multi_oneqtl}
\title{Perform multivariate, one-QTL model fitting for markers on all chromosomes}
\usage{
scan_multi_oneqtl(
  probs_list,
  pheno,
  kinship_list = NULL,
  addcovar = NULL,
  cores = 1
)
}
\arguments{
\item{probs_list}{an list of arrays of founder allele probabilities}

\item{pheno}{a matrix of phenotypes}

\item{kinship_list}{a list of kinship matrices, one for each chromosome}

\item{addcovar}{a matrix, n subjects by c additive covariates}

\item{cores}{number of cores for parallelization via parallel::mclapply()}
}
\value{
a tibble with d + 1 columns. First d columns indicate the genetic data (by listing the marker ids) used in the design matrix; last is log10 likelihood
}
\description{
The function first discards individuals with one or more missing phenotypes or missing covariates.
It then infers variance components, Vg and Ve. Both Vg and Ve
are d by d covariance matrices. It uses an expectation maximization algorithm, as
implemented in the `gemma2` R package. `gemma2` R package is an R implementation of the
GEMMA algorithm for multivariate variance component estimation (Zhou & Stephens 2014 Nature methods).
Note that variance components are fitted on a model that uses the d-variate phenotype
but contains no genetic information. This model does, however,
use the specified covariates (after dropping dependent columns
in the covariates matrix).
These inferred covariance matrices, \eqn{\hat{Vg}} and \eqn{\hat{Ve}},
are then used in subsequent model fitting via
generalized least squares.
Generalized least squares model fitting is applied to every marker on
every chromosome.
For a single marker, we fit the model:
\deqn{vec(Y) = Xvec(B) + vec(G) + vec(E)} where
\deqn{G \sim MN(0, K, \hat{Vg})} and \deqn{E \sim MN(0, I, \hat{Ve})} where \eqn{MN} denotes the matrix-variate
normal distribution with three parameters: mean matrix, covariance among rows, and
covariance among columns. \eqn{vec} denotes the vectorization operation, ie, stacking by columns.
\eqn{K} is a kinship matrix, typically calculated by leave-one-chromosome-out methods.
\eqn{Y} is the n by d phenotypes matrix. \eqn{X} is a block-diagonal nd by fd matrix consisting of
d blocks each of dimension n by f. Each n by f block (on the diagonal) contains a matrix of
founder allele probabilities for the n subjects at a single marker. The off-diagonal blocks
have only zero entries.
The log-likelihood is returned for each model. The outputted object is a tibble with
d + 1 columns. The first d columns specify the markers used in the corresponding model fit, while
the last column specifies the log-likelihood value at that d-tuple of markers.
}
\examples{
# read data
n <- 50
pheno <- matrix(rnorm(2 * n), ncol = 2)
rownames(pheno) <- paste0("s", 1:n)
colnames(pheno) <- paste0("tr", 1:2)
probs <- array(dim = c(n, 2, 5))
probs[ , 1, ] <- rbinom(n * 5, size = 1, prob = 0.2)
probs[ , 2, ] <- 1 - probs[ , 1, ]
rownames(probs) <- paste0("s", 1:n)
colnames(probs) <- LETTERS[1:2]
dimnames(probs)[[3]] <- paste0("m", 1:5)
scan_multi_oneqtl(probs_list = list(probs, probs), pheno = pheno, cores = 1)

}
\references{
Knott SA, Haley CS (2000) Multitrait
least squares for quantitative trait loci detection.
Genetics 156: 899–911.

Jiang C, Zeng ZB (1995) Multiple trait analysis
of genetic mapping for quantitative trait loci.
Genetics 140: 1111-1127.

Zhou X, Stephens M (2014) Efficient multivariate linear
mixed model algorithms for genome-wide association studies.
Nature methods 11:407-409.

Broman KW, Gatti DM, Simecek P, Furlotte NA, Prins P, Sen S, Yandell BS, Churchill GA (2019)
R/qtl2: software for mapping quantitative trait loci with high-dimensional data and
multi-parent populations. GENETICS https://www.genetics.org/content/211/2/495.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/boot_pvl.R
\name{boot_pvl}
\alias{boot_pvl}
\title{Perform bootstrap sampling and calculate test statistic for each bootstrap sample}
\usage{
boot_pvl(
  probs,
  pheno,
  addcovar = NULL,
  kinship = NULL,
  start_snp = 1,
  n_snp,
  pleio_peak_index,
  nboot = 1,
  max_iter = 10000,
  max_prec = 1/1e+08,
  cores = 1
)
}
\arguments{
\item{probs}{founder allele probabilities three-dimensional array for one chromosome only (not a list)}

\item{pheno}{n by d matrix of phenotypes}

\item{addcovar}{n by c matrix of additive numeric covariates}

\item{kinship}{a kinship matrix, not a list}

\item{start_snp}{positive integer indicating index within probs for start of scan}

\item{n_snp}{number of (consecutive) markers to use in scan}

\item{pleio_peak_index}{positive integer index indicating genotype matrix for bootstrap sampling. Typically acquired by using `find_pleio_peak_tib`.}

\item{nboot}{number of bootstrap samples to acquire and scan}

\item{max_iter}{maximum number of iterations for EM algorithm}

\item{max_prec}{stepwise precision for EM algorithm. EM stops once incremental difference in log likelihood is less than max_prec}

\item{cores}{number of cores to use when calling mclapply to parallelize the bootstrap analysis.}
}
\value{
numeric vector of (log) likelihood ratio test statistics from `nboot_per_job` bootstrap samples
}
\description{
Create a bootstrap sample, perform multivariate QTL scan, and calculate log10 LRT statistic
}
\details{
Performs a parametric bootstrap method to calibrate test statistic values in the test of
pleiotropy vs. separate QTL. It begins by inferring parameter values at
the `pleio_peak_index` index value in the object `probs`. It then uses
these inferred parameter values in sampling from a multivariate normal
distribution. For each of the `nboot` sampled phenotype vectors, a two-dimensional QTL
scan, starting at the marker indexed by `start_snp` within the object
`probs` and extending for a total of `n_snp` consecutive markers. The
two-dimensional scan is performed via the function `scan_pvl_clean`. For each
two-dimensional scan, a log10 likelihood ratio test statistic is calculated. The
outputted object is a vector of `nboot` log10 likelihood ratio test
statistics from `nboot` distinct bootstrap samples.
}
\examples{

n <- 50
pheno <- matrix(rnorm(2 * n), ncol = 2)
rownames(pheno) <- paste0("s", 1:n)
colnames(pheno) <- paste0("tr", 1:2)
probs <- array(dim = c(n, 2, 5))
probs[ , 1, ] <- rbinom(n * 5, size = 1, prob = 0.2)
probs[ , 2, ] <- 1 - probs[ , 1, ]
rownames(probs) <- paste0("s", 1:n)
colnames(probs) <- LETTERS[1:2]
dimnames(probs)[[3]] <- paste0("m", 1:5)
boot_pvl(probs = probs, pheno = pheno,
        start_snp = 1, n_snp = 5, pleio_peak_index = 3, nboot = 1, cores = 1)


}
\references{
Knott SA, Haley CS (2000) Multitrait
least squares for quantitative trait loci detection.
Genetics 156: 899–911.

Walling GA, Visscher PM, Haley CS (1998) A comparison of
bootstrap methods to construct confidence intervals in QTL mapping.
Genet. Res. 71: 171–180.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim1.R
\name{sim1}
\alias{sim1}
\title{Simulate a single multivariate data set consisting of n subjects and d phenotypes for each}
\usage{
sim1(X, B, Sigma)
}
\arguments{
\item{X}{design matrix (incorporating genotype probabilities from two loci), dn by df}

\item{B}{a matrix of allele effects, f rows by d columns}

\item{Sigma}{dn by dn covariance matrix}
}
\value{
a vector of length dn. The first n entries are for trait 1, the second n for trait 2, etc.
}
\description{
Simulate a single multivariate data set consisting of n subjects and d phenotypes for each
}
\examples{
n_mouse <- 20
geno <- rbinom(n = n_mouse, size = 1, prob = 1 / 2)
X <- gemma2::stagger_mats(geno, geno)
B <- matrix(c(1, 2), ncol = 2, nrow = 1)
sim1(X, B, Sigma = diag(2 * n_mouse))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/qtl2pleio-package.r
\docType{package}
\name{qtl2pleio-package}
\alias{qtl2pleio}
\alias{qtl2pleio-package}
\title{qtl2pleio.}
\description{
Testing pleiotropy vs. separate QTL in multiparental populations
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/fboehm/qtl2pleio}
  \item Report bugs at \url{https://github.com/fboehm/qtl2pleio/issues}
}

}
\author{
\strong{Maintainer}: Frederick J Boehm \email{frederick.boehm@gmail.com} (\href{https://orcid.org/0000-0002-1644-5931}{ORCID})

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/process_inputs.R
\name{process_inputs}
\alias{process_inputs}
\title{Process inputs to scan functions}
\usage{
process_inputs(
  probs,
  pheno,
  addcovar,
  kinship,
  n_snp = dim(probs)[3],
  start_snp = 1,
  max_iter = 10^4,
  max_prec = 1/10^8
)
}
\arguments{
\item{probs}{a three-dimensional array of founder allele probabilities}

\item{pheno}{a matrix of d trait values}

\item{addcovar}{a matrix of covariates}

\item{kinship}{a kinship matrix}

\item{n_snp}{number of markers}

\item{start_snp}{index number of start position in the probs object.}

\item{max_iter}{max number of iterations for EM}

\item{max_prec}{max precision for stopping EM}
}
\description{
Process inputs to scan functions
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{make_id2keep}
\alias{make_id2keep}
\title{Identify shared subject ids among all inputs: covariates, allele probabilities array, kinship, and phenotypes}
\usage{
make_id2keep(probs, pheno, addcovar = NULL, kinship = NULL)
}
\arguments{
\item{probs}{an allele probabilities array}

\item{pheno}{a phenotypes matrix}

\item{addcovar}{a covariates matrix}

\item{kinship}{a kinship matrix}
}
\value{
a character vector of subject IDs common to all (non-null) inputs
}
\description{
We consider only those inputs that are not NULL. We then use `intersect` on pairs
of inputs' rownames to identify those subjects are shared among all non-NULL inputs.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/find_pleio_peak_tib.R
\name{find_pleio_peak_tib}
\alias{find_pleio_peak_tib}
\title{Find the marker index corresponding to the peak of the pleiotropy trace in a tibble where the last column contains log likelihood values and the first d columns contain marker ids}
\usage{
find_pleio_peak_tib(tib, start_snp)
}
\arguments{
\item{tib}{a (d+1) column tibble with first d columns containing marker ids and the last containing log likelihood values. Typically this is the output from `scan_pvl`.}

\item{start_snp}{positive integer, from the two-dimensional scan, that indicates where the scan started on the chromosome}
}
\value{
positive integer indicating marker index for maximum value of log lik under pleiotropy
}
\description{
Find the marker index corresponding to the peak of the pleiotropy trace in a tibble where the last column contains log likelihood values and the first d columns contain marker ids
}
\examples{
marker1 <- rep(paste0('SNP', 1:3), times = 3)
marker2 <- rep(paste0('SNP', 1:3), each = 3)
loglik <- runif(9, -5, 0)
tibble::tibble(marker1, marker2, loglik) -> tib
find_pleio_peak_tib(tib, start_snp = 1)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_pvl.R
\name{plot_pvl}
\alias{plot_pvl}
\title{Plot tidied results of a pvl scan}
\usage{
plot_pvl(
  dat,
  units = "Mb",
  palette = c("#999999", "#E69F00", "#56B4E9"),
  linetype = c("solid", "longdash", "dotted")
)
}
\arguments{
\item{dat}{a profile lod tibble}

\item{units}{a character vector of length one to indicate units for physical or genetic map}

\item{palette}{a character vector of length 3 containing strings for colors}

\item{linetype}{a character vector of length 3 specifying the linetype values for the 3 traces}
}
\value{
a ggplot object with profile LODs
}
\description{
Plot tidied results of a pvl scan
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calc_Bhat.R
\name{calc_Bhat}
\alias{calc_Bhat}
\title{Calculate estimated allele effects, B matrix}
\usage{
calc_Bhat(X, Sigma_inv, Y)
}
\arguments{
\item{X}{dn by df block-diagonal design matrix that incorporates genetic info for d markers. Note that we can use the same marker data twice.}

\item{Sigma_inv}{dn by dn inverse covariance matrix, often composed as the inverse of \eqn{K \otimes V_g + I_n \otimes V_e}}

\item{Y}{dn by 1 matrix, ie, a column vector, of d phenotypes' measurements}
}
\value{
a df by 1 matrix of GLS-estimated allele effects
}
\description{
Calculate estimated allele effects, B matrix
}
\examples{
X1 <- as.matrix(rbinom(n = 100, size = 1, prob = 1 / 2))
X <- gemma2::stagger_mats(X1, X1)
Sigma_inv <- diag(200)
Y <- runif(200)
calc_Bhat(X, Sigma_inv, Y)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scan_multi_onechr.R
\name{scan_multi_oneqtl_perm}
\alias{scan_multi_oneqtl_perm}
\title{Permute the phenotypes matrix and then scan the genome. Record the genomewide greatest LOD score for each permuted data set.}
\usage{
scan_multi_oneqtl_perm(
  probs_list,
  pheno,
  kinship_list = NULL,
  addcovar = NULL,
  n_perm = 1,
  cores = 1
)
}
\arguments{
\item{probs_list}{a list of founder allele probabilities, one array per chromosome}

\item{pheno}{a matrix of trait values}

\item{kinship_list}{a list of kinship matrices, one per chromosome}

\item{addcovar}{a matrix of covariate values}

\item{n_perm}{positive integer for the number of permuted data sets to scan.}

\item{cores}{number of cores for parallelization}
}
\value{
a vector of `n_perm` max lod statistics
}
\description{
Permute the phenotypes matrix and then scan the genome. Record the genomewide greatest LOD score for each permuted data set.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit1_pvl.R
\name{fit1_pvl}
\alias{fit1_pvl}
\title{Fit a model for a specified d-tuple of markers}
\usage{
fit1_pvl(indices, start_snp, probs, addcovar, inv_S, S, pheno)
}
\arguments{
\item{indices}{a vector of indices for extracting elements of `probs` array}

\item{start_snp}{an integer to specify the index of the marker where the scan - in call to scan_pvl - starts. This argument is needed because `mytab` has only relative indices (relative to the `start_snp` marker)}

\item{probs}{founder allele probabilities array}

\item{addcovar}{additive covariates matrix}

\item{inv_S}{inverse covariance matrix for the vectorized phenotype}

\item{S}{covariance matrix for the vectorized phenotype, ie, the inverse of inv_S. By making this a function input, we avoid inverting the matrix many many times.}

\item{pheno}{a n by d phenotypes matrix}
}
\value{
a number, the log-likelihood for the specified model
}
\description{
`fit1_pvl` uses several functions in the package qtl2pleio to fit the
linear mixed effects model for a single d-tuple of markers.
Creation of `fit1_pvl` - from code that originally resided in `scan_pvl`, enabled
parallelization via the `parallel` R package.
}
\examples{
n <- 50
pheno <- matrix(rnorm(2 * n), ncol = 2)
Vg <- diag(2)
Ve <- diag(2)
Sigma <- calc_Sigma(Vg, Ve, diag(n))
Sigma_inv <- solve(Sigma)
probs <- array(dim = c(n, 2, 5))
probs[ , 1, ] <- rbinom(n * 5, size = 1, prob = 0.2)
probs[ , 2, ] <- 1 - probs[ , 1, ]
mytab <- prep_mytab(d_size = 2, n_snp = 5)
fit1_pvl(mytab[1, ], start_snp = 1,
probs = probs, addcovar = NULL, inv_S = Sigma_inv,
S = Sigma,
pheno = pheno
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rcpp_log_dmvnorm2.R
\name{rcpp_log_dmvnorm2}
\alias{rcpp_log_dmvnorm2}
\title{Calculate log likelihood for a multivariate normal}
\usage{
rcpp_log_dmvnorm2(inv_S, mu, x, S)
}
\arguments{
\item{inv_S}{inverse covariance matrix}

\item{mu}{mean vector}

\item{x}{data vector}

\item{S}{covariance matrix, ie, the inverse of inv_S}
}
\description{
Calculate log likelihood for a multivariate normal
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_effects.R
\name{get_effects}
\alias{get_effects}
\title{Extract founder allele effects at a single marker from output of qtl2::scan1coef}
\usage{
get_effects(marker_index, allele_effects_matrix, map, columns = 1:8)
}
\arguments{
\item{marker_index}{an integer indicating where in the `map` object the peak position (or position of interest) is located}

\item{allele_effects_matrix}{output of `qtl2::scan1coef` for a single chromosome}

\item{map}{a map object for the chromosome of interest}

\item{columns}{which columns to choose within the `allele_effects_matrix`. Default is 1:8 to reflect 8 founder alleles of Diversity Outbred mice}
}
\value{
a vector of 8 founder allele effects at a single marker

a vector of founder allele effects at a single marker
}
\description{
Extract founder allele effects at a single marker from output of qtl2::scan1coef
}
\examples{
# set up allele effects matrix
ae <- matrix(dat = rnorm(100 * 8), ncol = 8, nrow = 100)
ae[, 8] <- - rowSums(ae[, 1:7])
colnames(ae) <- LETTERS[1:8]
rownames(ae) <- paste0(1, "_", 1:100)
# set up map
map <- 1:100
names(map) <- rownames(ae)
# call get_effects
get_effects(marker_index = 15, allele_effects_matrix = ae, map = map)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scan_multi_onechr.R
\name{scan_multi_onechr}
\alias{scan_multi_onechr}
\title{Perform multivariate, one-QTL model fitting for markers on one chromosome}
\usage{
scan_multi_onechr(
  probs,
  pheno,
  kinship = NULL,
  addcovar = NULL,
  start_snp = 1,
  n_snp = dim(probs)[3],
  max_iter = 10000,
  max_prec = 1/1e+08,
  cores = 1
)
}
\arguments{
\item{probs}{an array of founder allele probabilities for a single chromosome}

\item{pheno}{a matrix of phenotypes}

\item{kinship}{a kinship matrix for one chromosome}

\item{addcovar}{a matrix, n subjects by c additive covariates}

\item{start_snp}{index of where to start the scan within probs}

\item{n_snp}{the number of (consecutive) markers to include in the scan}

\item{max_iter}{maximum number of iterations for EM algorithm}

\item{max_prec}{stepwise precision for EM algorithm. EM stops once incremental difference in log likelihood is less than max_prec}

\item{cores}{number of cores for parallelization}
}
\value{
a tibble with d + 1 columns. First d columns indicate the genetic data (by listing the marker ids) used in the design matrix; last is log10 likelihood
}
\description{
`scan_multi_onechr` calculates log likelihood for d-variate phenotype model fits. Inputted parameter `start_snp` indicates where in the `probs` object to start the scan.
}
\examples{
# read data
n <- 50
pheno <- matrix(rnorm(2 * n), ncol = 2)
rownames(pheno) <- paste0("s", 1:n)
colnames(pheno) <- paste0("tr", 1:2)
probs <- array(dim = c(n, 2, 5))
probs[ , 1, ] <- rbinom(n * 5, size = 1, prob = 0.2)
probs[ , 2, ] <- 1 - probs[ , 1, ]
rownames(probs) <- paste0("s", 1:n)
colnames(probs) <- LETTERS[1:2]
dimnames(probs)[[3]] <- paste0("m", 1:5)
scan_multi_onechr(probs = probs, pheno = pheno, kinship = NULL, cores = 1)

}
\references{
Knott SA, Haley CS (2000) Multitrait
least squares for quantitative trait loci detection.
Genetics 156: 899–911.

Jiang C, Zeng ZB (1995) Multiple trait analysis
of genetic mapping for quantitative trait loci.
Genetics 140: 1111-1127.

Zhou X, Stephens M (2014) Efficient multivariate linear
mixed model algorithms for genome-wide association studies.
Nature methods 11:407-409.

Broman KW, Gatti DM, Simecek P, Furlotte NA, Prins P, Sen S, Yandell BS, Churchill GA (2019)
R/qtl2: software for mapping quantitative trait loci with high-dimensional data and
multi-parent populations. GENETICS https://www.genetics.org/content/211/2/495.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calc_covs.R
\name{calc_covs}
\alias{calc_covs}
\title{Calculate Vg and Ve from d-variate phenotype and kinship}
\usage{
calc_covs(
  pheno,
  kinship,
  X1pre = rep(1, nrow(kinship)),
  max_iter = 1e+06,
  max_prec = 1/1e+08,
  covariates = NULL
)
}
\arguments{
\item{pheno}{n by d matrix of phenotypes}

\item{kinship}{a kinship matrix, n by n}

\item{X1pre}{n by c design matrix. c = 1 to ignore genotypes}

\item{max_iter}{maximum number of EM iterations}

\item{max_prec}{maximum precision for stepwise increments in EM algorithm}

\item{covariates}{a n by n.cov matrix of numeric covariates}
}
\value{
a list with 2 named components, Vg and Ve. Each is a d by d covariance matrix.
}
\description{
Calculate Vg and Ve from d-variate phenotype and kinship
}
\examples{
calc_covs(pheno = matrix(data = rnorm(100), nrow = 50, ncol = 2), kinship = diag(50))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{check_missingness}
\alias{check_missingness}
\title{Check for missingness in phenotypes or covariates}
\usage{
check_missingness(input_matrix)
}
\arguments{
\item{input_matrix}{phenotypes or covariates matrix}
}
\value{
character vector of subjects that have no missingness
}
\description{
We use `is.finite` from base R to identify those subjects that have one or
more missing values in `input_matrix`. We then return a character vector of subjects
that have no missingness in `input_matrix`.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calc_lrt_tib.R
\name{calc_lrt_tib}
\alias{calc_lrt_tib}
\title{Calculate a likelihood ratio test statistic from the output of scan_pvl()}
\usage{
calc_lrt_tib(scan_pvl_out)
}
\arguments{
\item{scan_pvl_out}{outputted tibble from scan_pvl}
}
\value{
a number, the (log) likelihood ratio test statistic
}
\description{
Calculate a likelihood ratio test statistic from the output of scan_pvl()
}
\examples{
rep(paste0('Marker', 1:3), times = 3) -> marker1
rep(paste0('Marker', 1:3), each = 3) -> marker2
runif(9, -1, 0) -> ll
tibble::tibble(marker1, marker2, ll) -> scan_out
calc_lrt_tib(scan_out)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prep_mytab.R
\name{prep_mytab}
\alias{prep_mytab}
\title{Prepare mytab object for use within scan_pvl R code}
\usage{
prep_mytab(d_size, n_snp, pvl = TRUE)
}
\arguments{
\item{d_size}{an integer, the number of traits}

\item{n_snp}{an integer, the number of markers}

\item{pvl}{logical indicating whether to output dataframe with all d-tuples for a d-QTL scan, or only those models that examine one marker at a time.}
}
\value{
a data.frame with d_size + 1 columns and (n_snp)^d_size rows. Last column is NA and named loglik.
}
\description{
Prepare mytab object for use within scan_pvl R code
}
\examples{
prep_mytab(2, 10)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/convert_to_scan1_output.R
\name{convert_to_scan1_output}
\alias{convert_to_scan1_output}
\title{Convert `scan_multi_oneqtl` output of `qtl2::scan1` output}
\usage{
convert_to_scan1_output(sm_output, trait_name)
}
\arguments{
\item{sm_output}{tibble output from scan_multi_oneqtl for one chromosome only}

\item{trait_name}{character vector (of length one) specifying the trait names}
}
\value{
object of class `scan1`
}
\description{
We convert output of `scan_multi_oneqtl` into format outputted by `qtl2::scan1`.
}
\examples{
# read data
iron <- qtl2::read_cross2(system.file("extdata", "iron.zip", package="qtl2"))
\dontshow{iron <- iron[,7]}

# insert pseudomarkers into map
map <- qtl2::insert_pseudomarkers(iron$gmap, step=1)

# calculate genotype probabilities
probs <- qtl2::calc_genoprob(iron, map, error_prob=0.002)

# grab phenotypes and covariates; ensure that covariates have names attribute
pheno <- iron$pheno
covar <- match(iron$covar$sex, c("f", "m")) # make numeric
names(covar) <- rownames(iron$covar)
Xcovar <- qtl2::get_x_covar(iron)

aprobs <- qtl2::genoprob_to_alleleprob(probs)
sm_out <- scan_multi_oneqtl(probs = aprobs, pheno = pheno)
sm_to_s1 <- convert_to_scan1_output(sm_out[[1]], trait_name = "tr1and2")

# 95\% Bayes credible interval for QTL on chr 7, first phenotype
qtl2::bayes_int(sm_to_s1, map)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{subset_input}
\alias{subset_input}
\title{Subset an input object - allele probabilities array or phenotypes matrix or covariates matrix. Kinship has its own subset function}
\usage{
subset_input(input, id2keep)
}
\arguments{
\item{input}{a matrix of either phenotypes or covariates or array of allele probabilities}

\item{id2keep}{a character vector of subject ids to identify those subjects that are shared by all inputs}
}
\value{
an object resulting from subsetting of `input`. Its rows are ordered per `id2keep`
}
\description{
An inputted matrix or 3-dimensional array is first subsetted - by rownames - to remove
those subjects who are not in `id2keep`. After that, the object's rows
are ordered to match the ordering of subject ids in the vector `id2keep`. This
(possibly reordered) object is returned.
}
\examples{
# define s_id
s_id <- paste0("s", 1:10)
# set up input matrix
foo <- matrix(data = rnorm(10 * 3), nrow = 10, ncol = 3)
rownames(foo) <- s_id
subset_input(input = foo, id2keep = s_id)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calc_profile_lods.R
\name{calc_profile_lods}
\alias{calc_profile_lods}
\title{Calculate profile lods for all traits}
\usage{
calc_profile_lods(scan_pvl_out)
}
\arguments{
\item{scan_pvl_out}{tibble outputted from scan_pvl}
}
\value{
a tibble with 3 columns, indicating 'marker identity, trace (pleiotropy or profile1, profile2, etc.), and value of the profile lod (base 10) for that trace at that marker.
}
\description{
Calculate profile lods for all traits
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calc_sqrt_mat.R
\name{calc_invsqrt_mat}
\alias{calc_invsqrt_mat}
\title{Calculate matrix inverse square root for a covariance matrix}
\usage{
calc_invsqrt_mat(A)
}
\arguments{
\item{A}{covariance matrix}
}
\description{
Calculate matrix inverse square root for a covariance matrix
}
